unit u_printreport;
{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to 2016 (LAZARUS),             }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}


{$mode delphi}

interface

uses
  Forms, RLReport,
  RLPreviewForm, RLPDFFilter, RLRichFilter,
  u_ancestroviewer;

type

  { TPrintGraph }

  TPrintGraph = class(TForm)
    ps: TRLPreviewSetup;
    RLPDFFilter: TRLPDFFilter;
    RLRichFilter: TRLRichFilter;
    rp: TRLReport;
    procedure rpBeforePrint(Sender: TObject; var PrintIt: boolean);
    procedure DataModuleCreate(Sender: TObject);
  private
    FGraph: TGraphComponent;
    FGraphForm: TCustomForm;
    //les pages
    FTopDecalX,
    FTopDecalY,
    fDecalagePapierY: extended;
    fDecalagePapierX: extended;
    procedure PrintAPage(const nx,ny: integer; const OnePage : Boolean = False );
    procedure PrintSave(const APrintOut: integer; const ASFilename: String='');

  public
    fpriorreport : TRLReport;
    { public declarations }
    property DecalagePapierX: extended read fDecalagePapierX write fDecalagePapierX;
    property DecalagePapierY: extended read fDecalagePapierY write fDecalagePapierY;

    procedure PreparePages(const aGraphForm: TCustomForm;const aGraphComponent: TGraphComponent; const OnePage :boolean = False);
    procedure Print(const aGraphForm: TCustomForm;const aGraphComponent: TGraphComponent; const APrintOut: integer; const ASFilename : String = '');
    procedure PrintPage(const aGraphForm: TCustomForm; const aGraphComponent: TGraphComponent; const px, py: integer; const APrintOut: integer; const ASFilename : String = '');
    procedure PrinterSetup;
  end;

var
  PrintGraph: TPrintGraph=nil;
  ScaleHeight,ScaleWidth : double;

implementation

{$R *.lfm}

uses fonctions_reports, RLConsts, fonctions_system, u_common_graph_type;

{ TPrintGraph }


procedure TPrintGraph.rpBeforePrint(Sender: TObject; var PrintIt: boolean);
begin
  with rp.Margins,fGraph do
   Begin
    LeftMargin := rp.left;
    TopMargin := rp.Top;
    RightMargin := Graph_RecouvrementImpression;
    BottomMargin := Graph_RecouvrementImpression;

    Data.PreparePrint;
    Print(LeftMargin + fDecalagePapierX, TopMargin + fDecalagePapierY, rp.Width, rp.Height);
   end;
end;

procedure TPrintGraph.DataModuleCreate(Sender: TObject);
begin
  fGraph := nil;
end;
procedure TPrintGraph.PreparePages(const aGraphForm: TCustomForm;const aGraphComponent: TGraphComponent; const OnePage :boolean = False);
var nx: integer;
    ARect : TFloatRect;
Begin
  FGraphForm := aGraphForm;
  FGraph    := aGraphComponent;
  for nx := 0 to rp.ComponentCount - 1 do
    if rp.Components[nx] is TRLReport Then rp.Components[nx].destroy;
  fpriorreport :=nil;
  rp.NextReport:=nil;
  with rp do
    begin
    //  pages.PrepareUpdate;
   //   pages.BeginUpdate;
      Title := FGraphForm.Caption;
      FirstPageNumber:=1;
      with FGraph.Viewer,PageSetup do
        Begin
         ScaleHeight := MMAsPixels;
         ScaleWidth  := MMAsPixels;
        end;
      with aGraphComponent,Viewer do
       Begin
//        ScreenRatio:=MMAsPixels;
        BuildZonesImpression;
        PrepareGraph;
        GetRectEncadrement(ARect);
        FTopDecalY := -ARect.Top;
        FTopDecalX := -ARect.Left;
        if OnePage
         Then pages.LastPageNumber:= 1
         Else pages.LastPageNumber:= ( ViewYC.Count - 1 ) * ( ViewXC.Count - 1 );
       end;
    end;
end;

procedure TPrintGraph.Print(const aGraphForm: TCustomForm;const aGraphComponent: TGraphComponent; const APrintOut: integer; const ASFilename : String = '');
var
  areport  : TRLReport;
  nx, ny: integer;
begin
  rp.BeginDoc;
  try
    //les pages
    with aGraphComponent.Viewer do
      Begin
       for ny := 1 to ViewYC.Count - 1 do
          for nx := 1 to ViewXC.Count - 1 do
            PrintAPage(nx,ny);
      end;
  finally
    rp.EndDoc;
  end;

  PrintSave( APrintOut, ASFilename );
  if ASFilename > '' Then
    p_OpenFileOrDirectory(ASFilename);
end;
procedure TPrintGraph.PrintSave(const APrintOut: integer; const ASFilename : String = '');
var
  areport  : TRLReport;
  nx, ny: integer;
begin
  if APrintOut > 0
   Then rp.SaveToFile( ASFilename )
   Else rp.Print;
End;
procedure TPrintGraph.PrintAPage(const nx, ny: integer; const OnePage : Boolean = False );
var
  areport  : TRLReport;
begin
  //les pages
  with FGraph.Viewer do
   Begin
      //le décalage
      if  OnePage
      or (   ( ny = 1 )
         and ( nx = 1 ))
       then
         Begin
           areport := rp;
           areport.Surface.Clear;
           fpriorreport := areport;
         end
       Else
        Begin
          areport := frlr_CreateNewReport (rp);
          areport.PriorReport:=fpriorreport;
        end;
      fDecalagePapierX := -ViewXC[nx-1];
      fDecalagePapierY := -(ViewYC[ny-1] + FTopDecalY) + FTopDecalY;
      PrintCanvas := areport.Surface;
      areport.BeforePrint:=rpBeforePrint;
      areport.Prepare;
    end;
end;

procedure TPrintGraph.PrintPage(const aGraphForm: TCustomForm; const aGraphComponent: TGraphComponent;
  const px, py: integer; const APrintOut: integer; const ASFilename : String = '');
begin
  // avoir le bon numéro de page.
  PreparePages(aGraphForm,aGraphComponent,True);
  fGraph := aGraphComponent;
    //le décalage
  rp.BeginDoc;
  try
    //la page
    PrintAPage(px,py,True);
  finally
    rp.EndDoc;
  end;
  PrintSave(APrintOut,ASFilename);
end;

procedure TPrintGraph.PrinterSetup;
begin
  rp.ShowPrintDialog;
end;
end.
