unit u_connexion;
{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to 2016 (LAZARUS),             }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}


interface

uses IBDatabase,Forms, u_common_resources;

function ConnexionBase(Connexion:TIBDatabase;const NomBase:string;Transaction:TIBTransaction
  ;const lc_type:string
  ;var UserName,PassWord,RoleName:string
  ;const NomTableTest:string
  ;var UtilEstAdmin:Boolean //le test n'est effectué que si True au départ
  ;Proprio:TForm=nil):Boolean;

implementation

uses SysUtils,
  {$IFNDEF FPC}
   Windows,
  {$ELSE}
    LCLIntf, LCLType, LazFileUtils,
  {$ENDIF}
  IB,IBSQL,Controls,Dialogs,IdentFB,
  fonctions_string,
  fonctions_dialogs,
  fonctions_ibx,
  fonctions_dbobjects,
  u_genealogy_context,
  u_common_functions,u_common_const,
  process,
  fonctions_system;


function ConnexionBase(Connexion:TIBDatabase;const NomBase:string;Transaction:TIBTransaction
  ;const lc_type:string
  ;var UserName,PassWord,RoleName:string
  ;const NomTableTest:string
  ;var UtilEstAdmin:Boolean
  ;Proprio:TForm=nil):Boolean;

var
  Attrs,PosSep:integer;
  bContinue,bShift:boolean;
  q:TIBSQL;
  s:string;

  function AskIdentFB:boolean;
  var
    Id,Pw,Rl:string;
  begin
    Id:=UserName;
    Pw:=PassWord;
    Rl:=RoleName;
    Result:=False;
    if LoginDialogFB(NomBase,Id,Pw,Rl,Proprio) then
    begin
      Result:=True;
      UserName:=Id;
      PassWord:=Pw;
      RoleName:=Rl;
      setBaseParams ( Connexion,UserName,PassWord,RoleName,lc_type);
    end;
  end;

  function ActiveTransaction(var Merreur:string):boolean;
  begin
    Result:=True;
    try
      Transaction.DefaultDatabase:=Connexion;
      Transaction.StartTransaction;
      q:=TIBSQL.Create(Application);
      with q do
       Begin
        ParamCheck:=False;
        Database:=Connexion;
        Transaction:=Transaction;
        SQL.Text:='select first(1) * from '+NomTableTest;
        ExecQuery;
        CLose;
       end;
    except
      on E:EIBError do
      begin
        Merreur:=E.Message;
        q.Free;
        if Transaction.Active then
          Transaction.Rollback;
        if Connexion.Connected then
          Connexion.Close;
        Result:=False;
      end;
    end;
  end;

  function ActiveBaseReseau(var Merreur:string):boolean;
  var
    s:string;
  begin
    Result:=True;
    try
//      Connexion.
      Connexion.Open;
      if not ActiveTransaction(s) then
      begin
        Result:=False;
        Merreur:=rs_Error_You_are_identified_on_server_but_you_have_no_right_on_database+_CRLF+_CRLF
          +rs_Firebird_Message+_CRLF+s;
        ;
      end;
    except
      on E:EIBError do
      begin
        Result:=False;
        s:=rs_Error_Cannot_connect_to_database+_CRLF+_CRLF;
        if PosSep>2 then
          s:=s+fs_RemplaceMsg(rs_Error_Verify_that_your_connected_to_network_server_address_alias_password,
                 [copy(NomBase,1,PosSep-1),copy(NomBase,PosSep+1,100)])+_CRLF+_CRLF
        else if PosSep=0 then
          s:=s+fs_RemplaceMsg(rs_Error_Verify_that_database_alias_is_defined_in_alias_conf_file,[copy(NomBase,1,100)])+_CRLF+_CRLF
        else
          s:=s+rs_Error_Verify_that_its_file_exists+_CRLF+_CRLF+_CRLF;
        Merreur:=s+rs_Firebird_Message+_CRLF+E.Message;
      end;
    end;
  end;

  function EstAdmin:Boolean;
  begin
    Result:=False;
    with q do
    try
      if UtilEstAdmin then //test effectué uniquement si True au départ
      begin
        if UserName=UserName_defaut then
          Result:=True
        else
        try
          Close;
          SQL.Text:='select first(1) rdb$grantor from rdb$user_privileges where rdb$user=current_user';
          ExecQuery;
          if Fields[0].AsString=UserName then //propriétaire
            Result:=True;
        except
          //Result:=False;
        end;
      end;
    finally
      Free;
    end;
  end;

begin

  result:=True;
  bShift:=GetKeyState(VK_SHIFT)<0;
  Screen.Cursor:=crHourGlass;
  try
    if Transaction.Active then
      Transaction.Rollback;
    if Connexion.Connected then
      Connexion.Close;
    s:='';

    PosSep:=pos(':',NomBase);
    {$IFDEF WINDOWS}
    if ( PosSep=2 ) then //FB embedded et base locale
    {$ELSE}
    if (NomBase <> '' ) and  (NomBase[1] = '/' )  Then
    {$ENDIF}
    begin
      if not FileExistsUTF8(NomBase) then
      begin
        Screen.Cursor:=crDefault;
        MyMessageDlg(fs_RemplaceMsg(rs_Error_Database_file_doesnt_exist,[NomBase]),mmtError, [mmbOK],Proprio);
        result:=False;
        exit;
      end;

      Attrs:=FileGetAttrUTF8(NomBase);
      bContinue:=True;

      if (Attrs and faReadOnly)<>0 then
      begin
        Screen.Cursor:=crDefault;
        if MyMessageDlg(rs_Confirm_using_readonly_database,mmtError, mmbYesNo,Proprio)=mrYes then
        begin
          if MyMessageDlg(rs_Confirm_readonly_supress
            ,mmtError, mmbYesNo,Proprio)=mrYes then
            bContinue:=FileSetAttr(NomBase,Attrs-faReadOnly)=0
          else
            bContinue:=False;
        end
        else
          bContinue:=False;
      end;
      if bContinue=False then
      begin
        MyMessageDlg(rs_Error_Cannot_start_software_with_a_readonly_database,mmtError, [mmbOK],Proprio);
        result:=False;
        exit;
      end
      else
        Screen.Cursor:=crHourGlass;
    end;

    try
      Connexion.DatabaseName:=NomBase;
      if UserName = '' Then UserName:=UserName_defaut;
      if PassWord = '' Then PassWord:=Password_defaut;
      setBaseParams(Connexion,UserName,PassWord,RoleName,lc_type);
      {$IFDEF WINDOWS}
      if (FileExistsUTF8(ExtractFilePath(Application.ExeName)+'fbclient.dll')or FileExistsUTF8(ExtractFilePath(Application.ExeName)+'fbembed.dll'))and(PosSep in [0,2]) then //FB embedded et base locale
      {$ELSE}
      if (NomBase > '' ) and  (NomBase[1] = '/' )  Then
      {$ENDIF}
      begin//avec FB embedded, l'utilisateur est toujours SYSDBA et le mot de passe indifférent
        try
          Connexion.Open;
          if not ActiveTransaction(s) then
          begin
            Screen.Cursor:=crDefault;
            MyMessageDlg(rs_Error_Database_too_old_or_corrupted+rs_Firebird_Message+_CRLF+s,mmtError, [mmbOK],Proprio);
            Result:=False;
            exit;
          end;
          UserName:=UserName_defaut;
        except
          on E:EIBInterBaseError do
           Begin
             MyMessageDlg(rs_Error_Cannot_connect_to_database,e.Message);
             Screen.Cursor:=crDefault;
             result:=False;
             Exit;
           end;
          on E:EIBError do
          try
            AskIdentFB;
            Transaction.Active := True;
            Connexion.Open;
          except
            on E:EIBError do
             Begin
              s:=rs_Error_Cannot_connect_to_database+_CRLF+_CRLF;
              if PosSep=0 then //base locale définie par un alias
                s:=s+fs_RemplaceMsg(rs_Error_Verify_that_database_alias_is_defined_in_alias_conf_file,
                  [copy(NomBase,1,100)]);
              Screen.Cursor:=crDefault;
              MyMessageDlg(s+rs_Firebird_Message+_CRLF+E.Message,mmtError, [mmbOK],Proprio);
              result:=False;
              exit;
            end;
          end;
        end;
        if not Connexion.Connected Then
         begin
          Result:=False;
          Screen.Cursor:=crDefault;
          exit;
         end;
      end
      else
      begin
        if bShift then
          bShift:=AskIdentFB;
        if ActiveBaseReseau(s) then
        begin
          if not bShift then
          begin
            UserName:=UserName_defaut;
            RoleName:='';
          end;
        end
        else //n'accepte pas les valeurs par défaut
        begin
          if not bShift then //si bShift LoginDialogFB a déjà été présenté et accepté
          begin
            setBaseParams(Connexion,UserName,PassWord,RoleName,lc_type);
            if not ActiveBaseReseau(s) then //n'accepte pas les valeurs précédentes
            begin
              if AskIdentFB then
                ActiveBaseReseau(s);
            end;
          end;
        end;

        if not Connexion.Connected Then
         begin
          Result:=False;
          Screen.Cursor:=crDefault;
          // will retry with full db path name
          if gci_context.PathFileNameBdd <> CST_ANCESTROMANIA_DATABASE Then
            MyMessageDlg(s,mmtError, [mmbOK],Proprio);
          exit;
         end;
      end;
      UtilEstAdmin:=EstAdmin;
    except
      if Transaction.Active then
        Transaction.Rollback;
      if Connexion.Connected then
        Connexion.Close;
    end;
  finally
    Screen.Cursor:=crDefault;
  end;
end;

end.

