unit CompSuperForm;
{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to 2016 (LAZARUS),             }
{    Copyright Andr? Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestro 2013 GPL)           }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}


interface

{$IFDEF FPC}
{$Mode Delphi}
{$ENDIF}

uses
  Classes,
  Controls,
{$IFNDEF FPC}
  Windows,
{$ENDIF}
  extctrls,
  buttons,
  Forms;


type
  TFormMode = (sfmEdit, sfmConsult, sfmNew, sfmUnknown);

  TIncrustMode = (aicCenter, aicTopLeft, aicAllClient, aicAllHeight, aicCenterTop, aicAllWidth);

  { TSuperBox }

  TSuperBox = class(TScrollBox)
  public
    procedure Resize ; override;
  End;

  { TSuperPanel }

  TSuperPanel = class(TPanel)
  public
    procedure Resize ; override;
  End;

  { TSuperForm }

  TSuperForm = class(TForm)

  private
    FVisible : Boolean;
    fFormMode: TFormMode;
    FDefaultCloseAction: TCloseAction;
    fAutoFillFieldsWhenShow: boolean;
    fIncrustMode: TIncrustMode;
    fAdaptParentSize: boolean;
    fPanelParent: TWinControl;
    fFixedClientHeight: integer;
    fFixedClientWidth: integer;
    fAlreadyPassedInFirstActivate: boolean;
    fAlreadyPassedInFirstShow: boolean;
    fOnPrepare: TNotifyEvent;
    fDescription: string;
{$IFNDEF FPC}
    fShowBevel: boolean;

    procedure SetShowBevel(value: boolean);
{$ENDIF}
    procedure Incrust(ParentControl: TWinControl);
    procedure SetcsDisplayDragImage;

  protected
    procedure DoSetBounds(ALeft, ATop, AWidth, AHeight: integer); override;
    procedure DoShow; override;
    procedure Resize ; override;
    procedure DoClose(var CloseAction: TCloseAction); override;

{$IFNDEF FPC}
    procedure Paint; override;
{$ENDIF}
    procedure Activate; override;

  public
    procedure DoFirstActivate; virtual; // old event : read fOnFirstActivate write fOnFirstActivate;
    procedure DoFillFields; virtual; // old event : read fOnFillFields write fOnFillFields;
    procedure DoFillProperties; virtual; // old event : read fOnFillProperties write fOnFillProperties;
    procedure DoRefreshControls; virtual; // old event : read fOnRefreshControls write fOnRefreshControls;
    procedure DoShowFirstTime; virtual; // old event : read fOnShowFirstTime write fOnShowFirstTime;
    procedure DoPrepare; virtual; // old event : read fOnPrepare write fOnPrepare;
    constructor Create(AOwner: TComponent); override;
    procedure InitSuperForm; virtual;
    procedure ShowIncrust(ParentControl: TWinControl);virtual;
    procedure HideIncrust;virtual;
    procedure AlignFormInParent(sender: TObject);virtual;
    function ShowModal: Integer; override;
    property FormActivated: boolean read fAlreadyPassedInFirstActivate;

  published
    property FormMode: TFormMode read fFormMode write fFormMode default sfmUnknown;
    property AutoFillFieldsWhenShow: boolean read fAutoFillFieldsWhenShow write fAutoFillFieldsWhenShow;
    property IncrustMode: TIncrustMode read fIncrustMode write fIncrustMode default aicAllClient;
    property AdaptParentSize: boolean read fAdaptParentSize write fAdaptParentSize;

    property PanelParent: TWinControl read fPanelParent;
{$IFNDEF FPC}
    property ShowBevel: boolean read fShowBevel write SetShowBevel;
{$ENDIF}
    property Description: string read fDescription write fDescription;
    property DefaultCloseAction: TCloseAction read FDefaultCloseAction write FDefaultCloseAction default caFree;
  end;


implementation

constructor TSuperForm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  if not (csDesigning in ComponentState) Then
    Begin
      InitSuperForm;
      FDefaultCloseAction:=caFree;
      fIncrustMode:=aicAllClient;
      fFormMode:=sfmUnknown;
    end;
end;

procedure TSuperForm.InitSuperForm;
begin
  fFormMode := sfmEdit;
  fAdaptParentSize := true;
  fPanelParent := nil;
  fFixedClientHeight := ClientHeight;
  fFixedClientWidth := ClientWidth;
  fAlreadyPassedInFirstActivate := false;
  fAlreadyPassedInFirstShow := false;
end;


procedure TSuperForm.DoFillFields;
begin
end;


procedure TSuperForm.DoFillProperties;
begin
end;

procedure TSuperForm.DoRefreshControls;
begin
end;

procedure TSuperForm.DoShowFirstTime;
begin

end;




procedure TSuperForm.DoShow;
begin
  if not (csDesigning in ComponentState) Then
    Begin
      SetcsDisplayDragImage;

      if (not fAlreadyPassedInFirstShow) then
        begin
          DoShowFirstTime; ;
          fAlreadyPassedInFirstShow := true;
        end;

    end;

  inherited;

  if not (csDesigning in ComponentState) Then
    Begin
      if fAutoFillFieldsWhenShow
       then
        Begin
          DoFillFields;
        end;
      {$IFDEF LCLQT}
      if assigned ( OnActivate ) Then
       OnActivate ( Self );
      DoFirstActivate;
      fAlreadyPassedInFirstActivate:=True;
      {$ENDIF}
      {$IFDEF LCLQT5}
      if assigned ( OnActivate ) Then
       OnActivate ( Self );
      DoFirstActivate;
      fAlreadyPassedInFirstActivate:=True;
      {$ENDIF}
    end;
end;

procedure TSuperForm.Resize;
begin
 if not (csDesigning in ComponentState) Then
  Begin
   if not Assigned ( fPanelParent ) Then
    inherited Resize;
  end
 else
   inherited Resize;
end;

procedure TSuperForm.DoClose(var CloseAction: TCloseAction);
begin
  CloseAction:=FDefaultCloseAction;
  inherited DoClose(CloseAction);
  fAlreadyPassedInFirstActivate := False;
end;


procedure TSuperForm.Incrust(ParentControl: TWinControl);
begin
  case fIncrustMode of
    aicAllClient : Align := alClient;
    aicAllHeight : Align := alLeft;
    aicAllWidth : Align := alTop;
     aicCenter :
       Begin
        Align:=alNone;
        Top := (ParentControl.ClientHeight - Height) div 2;
        if Top  < 0 then Top  := 0;
       end;

  end;
  if fIncrustMode in [ aicCenter, aicCenterTop] Then
   Begin
    Align:=alNone;
    Left := (ParentControl.ClientWidth-Width) div 2;
    if Left < 0 then Left := 0;
   end;

  BorderStyle := bsNone;


  parent := ParentControl;
  fPanelParent := ParentControl;


  ClientHeight := fFixedClientHeight;
  ClientWidth := fFixedClientWidth;

end;

procedure TSuperForm.ShowIncrust(ParentControl: TWinControl);
begin
  Updating;
  Incrust(ParentControl);
  Updated;
  Show;
end;


procedure TSuperForm.HideIncrust;
begin
  Hide;
end;

procedure TSuperForm.AlignFormInParent(sender: TObject);
var Rect : TRect ;
  procedure p_SetToZero;
  Begin
    if Rect.Left < 0 Then Rect.Left := 0;
    if Rect.Top  < 0 Then Rect.Top  := 0;
  end;

begin
  if not assigned ( fPanelParent )
   Then
     Exit;
  case fIncrustMode of
    aicCenter:
      begin
        Rect.Right := Width ;
        Rect.Bottom := Height ;
        Rect.Left := (fPanelParent.ClientWidth - ClientWidth) div 2;
        Rect.Top :=  (fPanelParent.ClientHeight - ClientHeight) div 2;
        p_SetToZero;
        SetBounds(Rect.Left,Rect.Top,Rect.Right,Rect.Bottom);
      end;

    aicCenterTop:
      begin
        Rect.Right := Width ;
        Rect.Bottom := Height ;
        Rect.Top :=  (fPanelParent.ClientHeight - ClientHeight) div 2;
        p_SetToZero;
        SetBounds(Rect.Left,Rect.Top,Rect.Right,Rect.Bottom);
      end;

  end;
end;


{$IFNDEF FPC}
procedure TSuperForm.SetShowBevel(value: boolean);
begin
  fShowBevel := value;
  repaint;
end;
{$ENDIF}

{$IFNDEF FPC}
procedure TSuperForm.Paint;
begin
  inherited paint;
  if fShowBevel then DrawButtonFace(Canvas, ClientRect, 1, bsNew, True, False, False);
end;
{$ENDIF}


procedure TSuperForm.Activate;
begin
  inherited Activate;

  if not (csDesigning in ComponentState)
  and not fAlreadyPassedInFirstActivate then
    begin
      DoFirstActivate;
      fAlreadyPassedInFirstActivate := true;
    end;
end;

procedure TSuperForm.DoFirstActivate;
begin

end;

function TSuperForm.ShowModal: Integer;
begin
  if not (csDesigning in ComponentState) Then
    Begin
      if (not fAlreadyPassedInFirstShow) then
        begin
          DoShowFirstTime; ;
          fAlreadyPassedInFirstShow := true;
        end;

      if fAutoFillFieldsWhenShow then DoFillFields;

    end;

  result := inherited ShowModal;
end;

procedure TSuperForm.DoPrepare;
begin
  if assigned(fOnPrepare) then fOnPrepare(self);
end;


procedure TSuperForm.SetcsDisplayDragImage;
var
  n: integer;
begin
  ControlStyle := ControlStyle + [csDisplayDragImage];
  for n := 0 to ComponentCount - 1 do
    if (Components[n] is TControl)
      then TControl(Components[n]).ControlStyle := TControl(Components[n]).ControlStyle + [csDisplayDragImage];
end;

procedure TSuperForm.DoSetBounds(ALeft, ATop, AWidth, AHeight: integer);
begin
  if not (csDesigning in ComponentState) Then
   Begin
    if ( AWidth  < Width  ) Then HorzScrollBar.Position:=0;
    if ( AHeight < Height ) Then VertScrollBar.Position:=0;
   end;
  inherited DoSetBounds(ALeft, ATop, AWidth, AHeight);
end;

procedure AlignSuperForm ( const Control : TWinControl );
var li_i : Integer;
Begin
  with Control do
    Begin
      for li_i := 0 to ControlCount - 1 do
        if Controls [ li_i ] is TSuperForm Then
          ( Controls [ li_i ] as TSuperForm ).AlignFormInParent( Controls [ li_i ]);
    end;

end;

{ TSuperBox }

procedure TSuperBox.Resize;
begin
  inherited Resize;
  AlignSuperForm ( Self );
end;

{ TSuperPanel }

procedure TSuperPanel.Resize;
begin
  inherited Resize;
  AlignSuperForm ( Self );
end;

end.


