unit U_Form_Main;

{$IFDEF FPC}
{$mode Delphi}{$H+}
{$ENDIF}

interface

uses
  Classes, SysUtils, process, FileUtil, Forms, Controls, Graphics, Dialogs,
  ExtCtrls, StdCtrls, EditBtn, Buttons, IBDatabase, IBQuery, IBLocalDBSupport,
  U_OnFormInfoIni, u_ancestropictimages, u_buttons_speed, IBXCustomIBLocalDBSupport;

type

  { TFMain }

  TFMain = class(TForm)
    btnQuitter: TSBQuit;
    btnRestore: TSBImport;
    btnSauve: TSBExport;
    btnSelectBase: TSpeedButton;
    cbFichierLocal: TCheckBox;
    edNomBase: TComboBox;
    edNomSauv: TFileNameEdit;
    IATitle1: TIATitle;
    ibd_BASE: TIBDatabase;
    IBT_BASE: TIBTransaction;
    Image1: TImage;
    imInfos: TImage;
    Label1: TLabel;
    LabelBase: TLabel;
    MemoEntete: TMemo;
    MemoInfos: TMemo;
    OnFormInfoIni: TOnFormInfoIni;
    OpenDialog: TOpenDialog;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    Panel5: TPanel;
    PanInfos: TPanel;
    UIBQuery: TIBQuery;
    UIBRestore: TIBLocalDBSupport;
    procedure btnQuitterClick(Sender: TObject);
    procedure btnRestoreClick(Sender: TObject);
    procedure btnSauveClick(Sender: TObject);
    procedure btnSelectBaseClick(Sender: TObject);
    procedure edNomBaseCloseUp(Sender: TObject);
    procedure imInfosClick(Sender: TObject);
    procedure UIBBackupVerbose(Sender: TObject; Message: string);
    procedure UIBRestoreGetDatabaseName(Sender: TObject; var DBName: string);
    procedure UIBRestoreVerbose(Sender: TObject; Message: string);
  private
    { private declarations }
    Connected,
    isadmin  ,
    bEnCours : boolean;
    ControlesActifs:TList;
    Save_cursor:TCursor;
    function ConnexionBase(const Proprio: TForm; const NomBase : String): Boolean;
    function DllActive:string;
    function AdresseGbak:string;
    procedure DesActiveControls;
    procedure p_addItem;
    procedure ReActiveControls;
    function ExecuteCommande(const ChCommande:string):boolean;
 public
   procedure DoClose(var CloseAction: TCloseAction);override;
   procedure DoShow;override;
   constructor Create(Sender: TComponent);override;
    { public declarations }
  end; 

var
  FMain: TFMain;

implementation

{$R *.lfm}
Uses Registry,DynLibs,VersionSupport,fonctions_string,
     fonctions_dialogs,
     fonctions_components,
     fonctions_system,
     fonctions_ibx,
     IB,
     u_connexion,
     u_common_functions,
     u_common_const,
     u_restorestrings,LazFileUtils,
     LazUTF8;//,windows;

{ TFMain }

function TFMain.DllActive:string;
{$IFDEF WINDOWS}
const
  IBASE_DLL='gds32.dll';
  FIREBIRD_CLIENT='fbclient.dll';
  FIREBIRD_EMBEDDED='fbembedded.dll';
  HINSTANCE_ERROR=32;

var InstallDir: string;
    dllPathName: string;
    IBLibrary:TLibHandle;
{$ENDIF}
begin
{$IFDEF WINDOWS}
  Result:='';
  //d'abord voir si Firebird Embedded est installé dans le répertoire de l'application
  InstallDir:=ExtractFilePath(Application.ExeName);
  if FileExistsUTF8(InstallDir+FIREBIRD_EMBEDDED) then
  begin
    Result:=InstallDir+FIREBIRD_EMBEDDED
  end
  else //sinon voir si Firebird Client est dans le répertoire de l'application
  if FileExistsUTF8(InstallDir+FIREBIRD_CLIENT) then
  begin
    Result:=InstallDir+FIREBIRD_CLIENT;
  end
  else //sinon voir si Interbase Client est dans le répertoire de l'application
  if FileExistsUTF8(InstallDir+IBASE_DLL) then
  begin
    Result:=InstallDir+IBASE_DLL;
  end
  else//sinon chercher avec la clé de registre de Firebird serveur
  begin
    with TRegistry.Create do
    try
      RootKey:=HKEY_LOCAL_MACHINE;
      if OpenKeyReadOnly('SOFTWARE\Firebird Project\Firebird Server\Instances') then
      begin
        if ValueExists('DefaultInstance') then
        begin
          dllPathName:=ReadString('DefaultInstance')+'bin'+DirectorySeparator+FIREBIRD_CLIENT;
          if FileExists(dllPathName) then
          begin
            Result := dllPathName;
            Exit;
          end;
        end;
      end;
    finally
      Free;
    end;

    //si toujours pas trouvé, chercher si l'une des 2 librairies est dans le path
    //en essayant de la charger
    IBLibrary:=LoadLibrary(FIREBIRD_CLIENT);
    if IBLibrary>HINSTANCE_ERROR then // FIREBIRD_CLIENT trouvé dans le path
      Result:=FIREBIRD_CLIENT
    else
    begin //si FIREBIRD_CLIENT pas trouvé on regarde si IBASE_DLL est dans le path
      if IBLibrary<>NilHandle then
      begin //décharger la librairie si elle a été chargée
         UnloadLibrary(IBLibrary);
         IBLibrary:=0;
      end;
      if IBLibrary>HINSTANCE_ERROR then // IBASE_DLL trouvé dans le path
        Result:=IBASE_DLL;
    end;
    if IBLibrary<>NilHandle then
    begin //décharger la librairie si elle a été chargée
       UnloadLibrary(IBLibrary);
       IBLibrary:=0;
    end;
  end;
{$ELSE}
  Result:='libfbclient.so';
{$ENDIF}
end;

function TFMain.AdresseGbak:string;
const
  Gbak64='gbak'{$IFDEF CPU64}+'64'{$ENDIF}{$IFDEF WINDOWS}+'.exe'{$ENDIF};
  Gbak='gbak'{$IFDEF WINDOWS}+'.exe'{$ENDIF};
var InstallDir: string;
    exePathName: string;
    procedure SetInstallDir;
    Begin
      if FileExistsUTF8(InstallDir+Gbak64) then
        begin
          Result:=InstallDir+Gbak64;
          Exit;
        end;
      if FileExistsUTF8(InstallDir+Gbak) then
        begin
          Result:=InstallDir+Gbak;
          Exit;
        end;
    end;

begin
  Result:=Gbak;
  //d'abord voir si gbak.exe est installé dans le répertoire de l'application
  InstallDir:=ExtractFilePath(Application.ExeName);
  SetInstallDir;
  {$IFDEF WINDOWS}
  //sinon chercher avec la clé de registre de Firebird serveur
  with TRegistry.Create do
    try
      RootKey:=HKEY_LOCAL_MACHINE;
      if OpenKeyReadOnly('SOFTWARE\Firebird Project\Firebird Server\Instances') then
      begin
        if ValueExists('DefaultInstance') then
        begin
          exePathName:=ReadString('DefaultInstance')+'bin'+DirectorySeparator+Gbak;
          if FileExists(exePathName) then
          begin
            Result:=exePathName;
          end
        end;
      end;
    finally
      Free;
    end;
  {$ELSE}
  InstallDir:=ExtractFilePath('/usr/bin/');
  SetInstallDir;
  {$ENDIF}
end;

procedure TFMain.btnQuitterClick(Sender: TObject);
begin
  Close;
end;

procedure TFMain.p_addItem;
begin
  fb_AddToCombo(edNomBase,edNomBase.Text);
end;

procedure TFMain.btnRestoreClick(Sender: TObject);
var
  p:integer;
  BackFile,NomBase,PathNameDB,s,ChCommande:string;
begin
  if edNomBase.Text='' then
  begin
    MyMessageDlg(rb_database_not_defined
      ,mmtError,[mmbCancel],Self);
    Exit;
  end;
  if edNomSauv.Text='' then
  begin
    MyMessageDlg(rb_save_file_not_defined
      ,mmtError,[mmbCancel],Self);
    Exit;
  end;
  if MyMessageDlg(fs_RemplaceMsg(rb_confirm_you_asked_database_restoring,[edNomBase.Text,edNomSauv.Text])
      ,mmtConfirmation,mmbYesNo,Self)<>mrYes then
    Exit;
  DesActiveControls;
  bEnCours:=True;
  p_addItem;
  try
    MemoInfos.Clear;
    MemoInfos.Enabled:=True;
//    MemoInfos.SetFocus;
    BackFile:=Trim(UTF8ToSys(edNomSauv.Text));
    UIBRestore.Database:=ibd_BASE;
    NomBase:=UTF8ToSys(Trim(edNomBase.Text));
    ibd_BASE.Connected:=False;
    ibd_BASE.DatabaseName:=NomBase;
    {$IFDEF WINDOWS}
    p:=Pos(':',NomBase);
    {$ELSE}
    p:=Pos('/',NomBase);
    {$ENDIF}
    UIBRestore.DatabaseName:=NomBase;
    if Connected and isadmin then //base d'adresse locale exclusivement, pas de test si alias local
      begin
        PathNameDB:=ExtractFileDir(NomBase);
        try
          UIBRestore.RestoreDatabase(BackFile);
        except
          On E:Exception do
          begin
            MyMessageDlg(SysToUTF8(E.Message),mmtError,[mmbCancel],Self);
            exit;
          end;
        end;
      end
     else
      begin
        //traitement avec gback obligatoire
        ChCommande:=AdresseGbak;
        if ChCommande='' then
        begin
          MemoInfos.Append(rb_cannot_restore_gbak_is_out);
          exit;
        end;
        MemoInfos.Append(rb_Restoring_with_gbak);
        ChCommande:='"'+ChCommande+'" -R O "'+BackFile+'" "'+UTF8ToSys(Trim(edNomBase.Text))
                   +'" -USER SYSDBA -PASS masterkey -V';
        if not ExecuteCommande(ChCommande)then
        begin
          MemoInfos.Append(rb_errors_while_restoring);
          exit;
        end;
        {$IFNDEF WINDOWS}
        {$IFNDEF DARWIN}
        if ( p <> 1 )
        or ( pos ( '/var/', edNomBase.Text ) = 1 )
         then
          begin
            MemoInfos.Append(rb_setting_database_rights);

            ChCommande := fs_getSuperUserGUI + ' chown firebird.firebird "/var/lib/firebird/2.5/data/'+UTF8ToSys(Trim(edNomBase.Text))
                     +'" "/var/lib/firebird/3.0/data/'+UTF8ToSys(Trim(edNomBase.Text)) +'"';
            ExecuteCommande(ChCommande);
          end;
        {$ENDIF}
        {$ENDIF}
      end;
    MemoInfos.Append(rb_Restore_ended);
    {
    UIBQuery.SQL.Text:='select * from rdb$relation_fields where rdb$relation_name=''T_VERSION_BASE'''
                      +' and rdb$field_name=''OCTETS_FICHIER''';
    UIBQuery.Open;
    if not UIBQuery.Eof then
    begin
      UIBQuery.Close;
      UIBQuery.SQL.Text:='update T_VERSION_BASE set OCTETS_FICHIER=null';
      try
        UIBQuery.ExecSQL;
        MemoInfos.Append('Base Ancestromania réinitialisée.');
      except
        On E:Exception do
        begin
          MyMessageDlg(SysToUTF8(E.Message),mmtError,[mmbCancel],[],[fsBold],True,Self);
        end;
      end;
    end;
    UIBQuery.Close;
    UIBTransaction.CommitRetaining;}
  finally
    bEnCours:=False;
    ReActiveControls;
  end;
end;

function TFMain.ConnexionBase(const Proprio : TForm ; const NomBase : String):Boolean;
var lb_estadmin : Boolean;
Begin
  lb_estadmin := True;
  Result := u_connexion.ConnexionBase(ibd_BASE,NomBase,IBT_BASE,_lc_ctypeUTF8
    ,_user_name,_password,_role_name
    ,'T_VERSION_BASE',lb_estadmin,Proprio);
end;

procedure TFMain.btnSauveClick(Sender: TObject);
var
  p:integer;
  PathNameFBK,BackFile,NomBase,s,ChCommande:string;
begin
  if edNomBase.Text='' then
  begin
    MyMessageDlg(rb_database_not_defined,mmtError,[mmbCancel],Self);
    Exit;
  end;
  if edNomSauv.Text='' then
  begin
    MyMessageDlg(rb_save_file_not_defined,mmtError,[mmbCancel],Self);
    Exit;
  end;
  if MyMessageDlg(fs_RemplaceMsg(rb_confirm_you_asked_database_saving,[edNomBase.Text,edNomSauv.Text])
                 ,mmtConfirmation,mmbYesNo,Self)<>mrYes then
    Exit;
  p_addItem;
  DesActiveControls;
  bEnCours:=True;
  try
    MemoInfos.Clear;
    MemoInfos.Enabled:=True;
//    MemoInfos.SetFocus;

    NomBase:=UTF8ToSys(Trim(edNomBase.Text));
    {$IFDEF WINDOWS}
    p:=Pos(':',NomBase);
    {$ELSE}
    p:=Pos('/',NomBase);
    {$ENDIF}
    UIBRestore.Database.DatabaseName:=NomBase;
    BackFile:=Trim(UTF8ToSys(edNomSauv.Text));
    {$IFDEF WINDOWS}
    if p=2 then //base d'adresse locale exclusivement, pas de test si alias local
    {$ELSE}
    if p=1 then //base d'adresse locale exclusivement, pas de test si alias local
    {$ENDIF}
    begin //controle uniquement si alias local ou base locale ou fichiers de sauvegarde locaux
      PathNameFBK:=ExtractFileDir(BackFile);
      if not ForceDirectoriesUTF8(PathNameFBK) then
      begin
        MyMessageDlg(rb_cannot_save_directory_does_not_exists,mmtError,[mmbCancel],Self);
        Exit;
      end;
      try
        ConnexionBase(Self,NomBase);
         UIBRestore.SaveDatabase(BackFile);
      except
        On E:Exception do
        begin
          MyMessageDlg(SysToUTF8(E.Message),mmtError,[mmbCancel],Self);
          exit;
        end;
      end;
    end
   else
    begin
      //traitement avec gback obligatoire
      ChCommande:=AdresseGbak;
      if ChCommande='' then
      begin
        MemoInfos.Append(rb_cannot_save_gbak_is_out);
        exit;
      end;
      ChCommande:='"'+ChCommande+'" -B "'+UTF8ToSys(Trim(edNomBase.Text))+'" "'+BackFile+'" '
                 +'-USER SYSDBA -PAS masterkey -Z -V';
      MemoInfos.Append('Sauvegarde par Gbak.');
      if not ExecuteCommande(ChCommande)then
      begin
        MemoInfos.Append('Erreurs au cours de la sauvegarde.');
        exit;
      end;
    end;
    MemoInfos.Append('Sauvegarde terminée.');
  finally
    bEnCours:=False;
    ReActiveControls;
  end;
end;

procedure TFMain.btnSelectBaseClick(Sender: TObject);
begin
  OpenDialog.FileName:='';
  //préparation de la boite OpenDialog
  if DirectoryExistsUTF8(edNomBase.Text) then
  begin
    OpenDialog.InitialDir:=edNomBase.Text;
  end
  else
  begin
    if DirectoryExistsUTF8(ExtractFilePath(edNomBase.Text)) then
    begin
      OpenDialog.InitialDir:=ExtractFilePath(edNomBase.Text);
      if FileExistsUTF8(edNomBase.Text) then
        OpenDialog.FileName:=edNomBase.Text;
    end;
  end;

  if OpenDialog.Execute then
  begin
    edNomBase.Text:=OpenDialog.FileName;
    edNomBaseCloseUp(Self);
  end;
end;

procedure TFMain.edNomBaseCloseUp(Sender: TObject);
begin
  isadmin:=True;
  Connected:=False;
  if FileExistsUTF8(edNomBase.Text) Then
    Connected:= u_connexion.ConnexionBase(ibd_BASE,edNomBase.Text,IBT_BASE,_lc_ctype
    ,_user_name,_password,_role_name
    ,'T_VERSION_BASE',isadmin,Self);
  if not Connected
   Then
    MemoInfos.Lines.Add(rb_cannot_connect);
end;



procedure TFMain.DoClose(var CloseAction: TCloseAction);
begin
  Inherited;
  if bEnCours then
    CloseAction:=caNone;
end;

procedure TFMain.DoShow;
begin
  inherited;
  {$IFDEF LINUX}
  with edNomBase,Items do
   Begin
    if Count = 0 Then
      Add('Ancestromania.fdb');
    if ItemIndex = -1 Then
     ItemIndex:=0;
   end;
  UIBRestore.FirebirdDirectory:='/usr/bin';
  {$ENDIF}
  {$IFDEF WINDOWS}
  UIBRestore.EmptyDBArchive:='Ancestromania-updated.fdb';
  {$ELSE}
  UIBRestore.EmptyDBArchive:='Ancestromania-updated.fdb';
  {$ENDIF}
  MemoInfos.Clear;
  edNomBaseCloseUp(self);
end;

constructor TFMain.Create(Sender: TComponent);
var
  fIni:TRegistry;
  s,fKeyRegistry,NomLibrairie:string;
  i:integer;
begin
  Inherited;
  NomLibrairie:=DllActive;
  fIni:=TRegistry.create;
  try
    fIni.RootKey:=HKEY_CURRENT_USER;
    fKeyRegistry:='\SOFTWARE\Ancestromania\Path';
    if fIni.OpenKeyReadOnly(fKeyRegistry) then
    begin
      //La base de donnée
      edNomBase.Text:=SysToUTF8(fIni.ReadString('PathFileNameBdd'));
      fIni.CloseKey;
    end;
    fKeyRegistry:='\SOFTWARE\Ancestromania\Settings';
    if fIni.OpenKeyReadOnly(fKeyRegistry) then
    begin
      for i:=0 to edNomBase.DropDownCount-1 do
      begin
        s:=fIni.ReadString('NomBase'+IntToStr(i));
        if s>'' then
          edNomBase.Items.Add(SysToUTF8(s));
      end;
    end;
  finally
    fIni.Free;
  end;
  if pos(':',edNomBase.Text)=2 then
  begin
    s:=ChangeFileExt(ExtractFileName(edNomBase.Text),'.FBK');
    edNomSauv.Text:=ExtractFilePath(edNomBase.Text)+'backup\'+s;
  end;
  Caption:=fs_RemplaceMsg(rb_caption,[GetFileVersion]);
end;

procedure TFMain.imInfosClick(Sender: TObject);
begin
  MyMessageDlg(rb_Using,rb_infos,mmtInformation,[mmbOK],0,self);
end;

procedure TFMain.UIBBackupVerbose(Sender: TObject; Message: string);
begin
  MemoInfos.Append(UTF8Encode(Message));
  Application.ProcessMessages;
end;

procedure TFMain.UIBRestoreGetDatabaseName(Sender: TObject; var DBName: string);
begin
  DBName:=edNomBase.Text;
end;

procedure TFMain.UIBRestoreVerbose(Sender: TObject; Message: string);
begin
  MemoInfos.Append(UTF8Encode(Message));
  Application.ProcessMessages;
end;

procedure TFMain.DesActiveControls;
var
  i:integer;
begin
  Application.ProcessMessages;
  Save_Cursor:=Screen.Cursor;
  Screen.Cursor:=crHourglass;
  ControlesActifs:=TList.Create;
  for i:=0 to ComponentCount-1 do
  begin
    if Components[i] is TControl then
    begin
      if (Components[i] as TControl).Enabled=True then
      begin
        (Components[i] as TControl).Enabled:=False;
        ControlesActifs.Add(Components[i]);
      end;
    end;
  end;
end;

procedure TFMain.ReActiveControls;
var
  i:integer;
begin
  for i:=0 to ComponentCount-1 do
  begin
    if Components[i] is TControl then
    begin
      if ControlesActifs.IndexOf(Components[i])=-1 then
      begin
        if Components[i] is TRadioGroup then//bug des TRadioGroup, les boutons n'apparaissent
          (Components[i] as TRadioGroup).Enabled:=True;//pas désactivés alors qu'ils le sont.
        (Components[i] as TControl).Enabled:=False
      end
      else
        (Components[i] as TControl).Enabled:=True;
     end;
  end;
  ControlesActifs.Free;
  Screen.Cursor:=Save_Cursor;
end;

function TFMain.ExecuteCommande(const ChCommande:string):boolean;
const
  READ_BYTES = 2048;
var
  s:string;
  leProcess:TProcess;
  c:Char;
  M:TMemoryStream;
  BytesRead,n,i:Integer;
begin
  writeln ( ChCommande );
  MemoInfos.Append(fs_RemplaceMsg(rb_execute_command,[ChCommande]));
  Application.ProcessMessages;
  Result:=True;
  M:=TMemoryStream.Create;
  leProcess:=TProcess.Create(self);
  try
    leProcess.Options:=[poUsePipes,poStderrToOutPut];
    leProcess.CommandLine:=ChCommande;
    leProcess.ShowWindow:=swoHIDE;
    try
      leProcess.Execute;
      s:='';
      c:=#0;
      BytesRead:=0;
      while leProcess.Running do
      begin;
        M.SetSize(BytesRead+READ_BYTES);
        n:=leProcess.Output.Read((M.Memory+BytesRead)^,READ_BYTES);
        if n>0 then
        begin
          M.Seek(BytesRead,0);
          for i:=1 to n do
          begin
            M.ReadBuffer(c,1);
            if c=#10 then
            begin
              MemoInfos.Append(s);
              Application.ProcessMessages;
              s:='';
            end;
          end;
          Inc(BytesRead,n);
        end
        else
        begin
          sleep(100);
        end;
      end;
      repeat
        M.SetSize(BytesRead+READ_BYTES);
        n:=leProcess.Output.Read((M.Memory+BytesRead)^,READ_BYTES);
        if n>0 then
        begin
          M.Seek(BytesRead,0);
          for i:=1 to n do
          begin
            M.ReadBuffer(c,1);
            if c=#10 then
            begin
              MemoInfos.Append(s);
              Application.ProcessMessages;
              s:='';
            end;
          end;
          Inc(BytesRead,n);
        end;
      until n<=0;
      if s<>'' then
        MemoInfos.Append(s);
      if leProcess.ExitStatus>0 then
        Result:=False;
    except
      Result:=False
    end;
  finally
    leProcess.Free;
    M.Free;
  end;
end;

{$IFDEF FPC}
initialization
  OnGetLibraryName:= TOnGetLibraryName( p_setLibrary);
{$ENDIF}
end.

