{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_form_edit_raccourcis;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType,
{$ENDIF}
  U_FormAdapt,DB,
  U_ExtDBGrid,
  StdCtrls,Controls,Classes,ExtCtrls,
  u_buttons_appli,
  IBTable;

type

  { TFEditRaccourcis }

  TFEditRaccourcis=class(TF_FormAdapt)
    DataSource:TDataSource;
    dxDBGrid1: TExtDBGrid;
    pBorder:TPanel;
    Panel7:TPanel;
    Panel4:TPanel;
    Panel5:TPanel;
    Panel6:TPanel;
    btnAdd:TFWAdd;
    btnDel:TFWDelete;
    Label3:TLabel;
    
    btnFermer:TFWClose;
    btnImport:TFWImport;
    btnExport:TFWExport;
    Query:TIBTable;
{$IFNDEF FPC}
    QueryRAC_CLEF: TLongintField;
    QueryRAC_RAC: TIBStringField;
    QueryRAC_LIBELLE: TIBStringField;
{$ENDIF}
    procedure btnAddClick(Sender:TObject);
    procedure btnDelClick(Sender:TObject);

    procedure btnImportClick(Sender:TObject);
    procedure btnExportClick(Sender:TObject);
    procedure QueryBeforePost(DataSet:TDataSet);
    procedure QueryBeforeDelete(DataSet:TDataSet);
    procedure QueryAfterInsert(DataSet: TDataSet);
    procedure QueryAfterDelete(DataSet: TDataSet);
  private
    procedure OpenQuery;
  protected
    procedure ControlKeyDown(var Key:Word;
      Shift:TShiftState); override;
  public
    procedure Loaded; override;
    function CloseQuery:Boolean; override;
    procedure DoShowFirstTime; override;
    procedure DoRefreshControls; override;

  end;

implementation

uses u_dm,
    u_common_functions,
    u_common_ancestro,u_common_ancestro_functions,
    u_common_const,
    u_genealogy_context,
    u_form_export_table_2_textfile,
    u_form_Import_textfile_2_table,
    fonctions_dialogs,
    fonctions_string,
    fonctions_ini,
    Dialogs,
    SysUtils;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}

{ TFEditRefParticules }

procedure TFEditRaccourcis.Loaded;
begin
  f_GetMemIniFile();
  Panel7.Color:=gci_context.ColorLight;
  btnAdd.Color:=gci_context.ColorLight;
  btnDel.Color:=gci_context.ColorLight;
  btnAdd.ColorFrameFocus:=gci_context.ColorLight;
  btnDel.ColorFrameFocus:=gci_context.ColorLight;
  inherited;
end;

procedure TFEditRaccourcis.DoShowFirstTime;
begin
  caption:=rs_Caption_keyboard_shortcuts;
  OpenQuery;
end;

procedure TFEditRaccourcis.OpenQuery;
begin
  Query.DisableControls;
  try
    Query.Close;
    Query.Open;
  finally
    Query.EnableControls;
  end;
  doRefreshControls;
end;

procedure TFEditRaccourcis.DoRefreshControls;
begin
  btnAdd.enabled:=Query.Active;
  btnDel.enabled:=Query.Active and not Query.IsEmpty;
end;

procedure TFEditRaccourcis.btnAddClick(Sender:TObject);
begin
  Query.Insert;
  SendFocus(dxDBGrid1);
end;

procedure TFEditRaccourcis.QueryAfterInsert(DataSet: TDataSet);
begin
  doRefreshControls;
end;

procedure TFEditRaccourcis.btnDelClick(Sender:TObject);
begin
  Query.Delete;
end;

procedure TFEditRaccourcis.QueryBeforeDelete(DataSet:TDataSet);
begin
  if MyMessageDlg(rs_Do_you_delete_this_shortcut_input
    ,mmtConfirmation, mmbYesNo,Self)<>mrYes then
    Abort;
end;

procedure TFEditRaccourcis.QueryAfterDelete(DataSet: TDataSet);
begin
  doRefreshControls;
  SendFocus(dxDBGrid1);
end;

procedure TFEditRaccourcis.QueryBeforePost(DataSet:TDataSet);
var
  chaine:string;
  continue:boolean;
begin
  chaine:='';
  continue:=true;
  if DataSet.FieldByName('RAC_LIBELLE').AsString='' then
  begin
    continue:=false;
    chaine:='Le texte';
  end;
  if Trim(DataSet.FieldByName('RAC_RAC').AsString)='' then
  begin
    continue:=false;
    chaine:='L''abréviation';
  end;
  if continue then
  begin
    MyMessageDlg(fs_RemplaceMsg(rs_Error_must_not_to_be_empty,[chaine])
    ,mmtError,[mmbOk],Self);
    SendFocus(dxDBGrid1);
    Abort;
  end;
end;

function TFEditRaccourcis.CloseQuery:Boolean;
begin
  Result := inherited;
  if Result Then
   begin
    if Query.Active and(Query.State in [dsEdit,dsInsert])then
      Query.Post;
    dm.IBTrans_Secondaire.CommitRetaining;
    Query.Close;
    dm.LoadRaccourcis;
   end;
end;

procedure TFEditRaccourcis.ControlKeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  case Key of
    VK_ESCAPE:
      begin
        dm.IBTrans_Secondaire.RollbackRetaining;
        OpenQuery;
      end;
    _KEY_HELP:p_ShowHelp(_ID_EDIT_RACCOURCIS_SAISIE);
  end;
end;

procedure TFEditRaccourcis.btnImportClick(Sender:TObject);
var
  aFImportTextFile2Table:TFImportTextFile2Table;
begin
  aFImportTextFile2Table:=TFImportTextFile2Table.create(self);
  try
    aFImportTextFile2Table.TableName:='REF_RACCOURCIS';
    aFImportTextFile2Table.NameGeneratorPrimaryIndex:='GEN_REF_RACCOURCIS';
    aFImportTextFile2Table.NamePrimaryIndex:='RAC_CLEF';
    aFImportTextFile2Table.ParamsTable.Add('RAC_CLEF');
    aFImportTextFile2Table.ParamsTable.Add('RAC_RAC');
    aFImportTextFile2Table.ParamsTable.Add('RAC_LIBELLE');
    aFImportTextFile2Table.chbVider.Checked:=true;
    if aFImportTextFile2Table.ShowModal=mrOk then
      OpenQuery;
  finally
    aFImportTextFile2Table.free;
  end;
end;

procedure TFEditRaccourcis.btnExportClick(Sender:TObject);
var
  aFExportTable2TextFile:TFExportTable2TextFile;
begin
  aFExportTable2TextFile:=TFExportTable2TextFile.create(self);
  try
    aFExportTable2TextFile.TableName:='REF_RACCOURCIS';
    aFExportTable2TextFile.PropositionFileNameExport:='REF_RACCOURCIS.txt';
    aFExportTable2TextFile.ShowModal;
  finally
    aFExportTable2TextFile.Free;
  end;
end;

end.

