{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_form_edit_ref_particules;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType,
{$ENDIF}
  U_FormAdapt,  Forms,
  DB, U_ExtDBGrid,
  StdCtrls, Controls, Classes, ExtCtrls,
   u_buttons_appli,
     IBTable;

type
  TFEditRefParticules = class(TF_FormAdapt)
    DataSource: TDataSource;
    pBorder: TPanel;
    Panel7: TPanel;
    Panel4: TPanel;
    lbTitle: TLabel;
    Panel8: TPanel;
    Panel6: TPanel;
    Panel1: TPanel;
    dxDBGrid1: TExtDBGrid;
    btnAdd: TFWAdd;
    btnDel: TFWDelete;
    
    btnImport: TFWImport;
    btnExport: TFWExport;
    btnFermer: TFWClose;
    Query: TIBTable;
{$IFNDEF FPC}
    QueryPART_CLEF: TLongintField;
    QueryPART_LIBELLE: TStringField;
{$ENDIF}
    procedure btnAddClick(Sender: TObject);
    procedure btnDelClick(Sender: TObject);
    procedure btnExportClick(Sender: TObject);
    procedure btnImportClick(Sender: TObject);
    procedure QueryAfterInsert(DataSet: TDataSet);
    procedure QueryAfterDelete(DataSet: TDataSet);
    procedure QueryBeforePost(DataSet: TDataSet);
    procedure QueryBeforeDelete(DataSet: TDataSet);
  private
    procedure OpenQuery;
  protected
    procedure ControlKeyDown(var Key:Word;
      Shift:TShiftState); override;
    procedure DoClose(var Action: TCloseAction); override;
  public
    procedure Loaded; override;
    procedure DoShowFirstTime; override;
    procedure DoRefreshControls; override;

  end;

implementation

uses u_dm,
    u_common_functions,
    u_common_ancestro,
    fonctions_dialogs,
    u_common_const,
    u_common_ancestro_functions,
    u_form_export_table_2_textfile,
    u_form_Import_textfile_2_table,
    u_genealogy_context,
    fonctions_ini,
    Dialogs,
    SysUtils;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}

{ TFEditRefParticules }

procedure TFEditRefParticules.Loaded;
begin
  f_GetMemIniFile();
  Panel7.Color:=gci_context.ColorLight;
  btnAdd.Color:=gci_context.ColorLight;
  btnDel.Color:=gci_context.ColorLight;
  btnAdd.ColorFrameFocus:=gci_context.ColorLight;
  btnDel.ColorFrameFocus:=gci_context.ColorLight;
  inherited;
end;

procedure TFEditRefParticules.DoShowFirstTime;
begin
  OpenQuery;
  Caption:=rs_Caption_Refering_particles;
end;

procedure TFEditRefParticules.OpenQuery;
begin
  Query.DisableControls;
  try
    Query.Close;
    Query.Open;
  finally
    Query.EnableControls;
  end;
  doRefreshControls;
end;

procedure TFEditRefParticules.DoRefreshControls;
begin
  btnAdd.enabled := Query.Active;
  btnDel.enabled := Query.Active and not Query.IsEmpty;
end;

procedure TFEditRefParticules.btnAddClick(Sender: TObject);
begin
  Query.Insert;
  SendFocus(dxDBGrid1);
end;

procedure TFEditRefParticules.QueryAfterInsert(DataSet: TDataSet);
begin
  doRefreshControls;
end;

procedure TFEditRefParticules.btnDelClick(Sender: TObject);
begin
  Query.Delete;
end;

procedure TFEditRefParticules.QueryBeforeDelete(DataSet: TDataSet);
begin
  if MyMessageDlg(rs_Confirm_deleting_this_particle
    ,mmtConfirmation,[mmbYes,mmbNo],Self)<>mrYes then
    Abort;
end;

procedure TFEditRefParticules.QueryAfterDelete(DataSet: TDataSet);
begin
  doRefreshControls;
  SendFocus(dxDBGrid1);
end;

procedure TFEditRefParticules.QueryBeforePost(DataSet: TDataSet);
begin
  with DataSet do
  if Trim(FieldByName('PART_LIBELLE').AsString)='' then
  begin
    MyMessageDlg(rs_Error_Particle_not_to_be_empty_or_spaces_filled
    ,mmtError,[mmbOk],Self);
    SendFocus(dxDBGrid1);
    Abort;
  end;
end;

procedure TFEditRefParticules.DoClose(var Action: TCloseAction);
begin
  inherited;
  if Query.Active and (Query.State in [dsEdit, dsInsert]) then
    Query.Post;
  dm.IBTrans_Secondaire.CommitRetaining;
  Query.Close;
end;

procedure TFEditRefParticules.ControlKeyDown(
  var Key: Word; Shift: TShiftState);
begin
  inherited;
  case Key of
    VK_ESCAPE:
    begin
      dm.IBTrans_Secondaire.RollbackRetaining;
      OpenQuery;
    end;
    _KEY_HELP: p_ShowHelp(_ID_EDIT_TABLE_REF_PARTICULES);
  end;
end;

procedure TFEditRefParticules.btnExportClick(Sender: TObject);
var
  aFExportTable2TextFile: TFExportTable2TextFile;
begin
  aFExportTable2TextFile := TFExportTable2TextFile.create(self);
  try
    aFExportTable2TextFile.TableName := 'REF_PARTICULES';
    aFExportTable2TextFile.PropositionFileNameExport := 'Particules.txt';
    aFExportTable2TextFile.ShowModal;
  finally
    aFExportTable2TextFile.Free;
  end;
end;

procedure TFEditRefParticules.btnImportClick(Sender: TObject);
var
  aFImportTextFile2Table: TFImportTextFile2Table;
begin
  aFImportTextFile2Table := TFImportTextFile2Table.create(self);
  try
    aFImportTextFile2Table.TableName := 'REF_PARTICULES';
    aFImportTextFile2Table.NamePrimaryIndex := 'PART_CLEF';
    aFImportTextFile2Table.NameGeneratorPrimaryIndex := 'GEN_REF_PARTICULES';
    aFImportTextFile2Table.ParamsTable.Add('PART_CLEF');
    aFImportTextFile2Table.ParamsTable.Add('PART_LIBELLE');
    aFImportTextFile2Table.chbVider.Checked:=true;
    if aFImportTextFile2Table.ShowModal = mrOk then
      OpenQuery;
  finally
    aFImportTextFile2Table.free;
  end;
end;

end.
