unit u_Form_Histoire;
{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}


{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLIntf,
{$ENDIF}
  SysUtils, Variants, Classes, Graphics, Controls, Forms, U_FormAdapt,
  Dialogs, DB,
  u_framework_components,
  U_ExtDBImage, u_extdbgrid, ExtCtrls,
  u_framework_dbcomponents, u_ancestropictbuttons,
  u_buttons_appli, U_OnFormInfoIni, IBQuery, IBUpdateSQL,
  Menus, IBDatabase, DBCtrls;

type

  { TFHistoire }
  TFHistoire = class(TF_FormAdapt)
    btnAjout: TFWAdd;
    btnCategorie: TFWFolder;
    btnDelete: TFWDelete;
    btnExport: TFWExport;
    btnHistoire: TXAHistory;
    btnImport: TFWImport;
    cxDBImage1: TExtDBImage;
    cxLabel2: TFWLabel;
    Date: TDBText;
    Evenement: TDBText;
    dsHistoire: TDataSource;
    IBUpdateSQL1: TIBUpdateSQL;
    OnFormInfoIni: TOnFormInfoIni;
    Panel2: TPanel;
    Panel3: TPanel;
    pListe: TPanel;
    cxGrid1: TExtDBGrid;
    pPhotoDate: TPanel;
    ibHistoire: TIBQuery;
    Panel1: TPanel;
    PanelTexte: TPanel;
    Splitter1: TSplitter;
    uHistoire: TIBUpdateSQL;
    cxDBMemo1: TFWDBMemo;
    ibHistoireHI_ID: TLongintField;
    ibHistoireHI_DOSSIER: TLongintField;
    ibHistoireHI_DICORIGINE: TStringField;
    ibHistoireHI_DATE_TEXTE: TStringField;
    ibHistoireHI_CAT: TLongintField;
    ibHistoireHI_TITRE: TStringField;
    ibHistoireHI_DATE_CODE_DEBUT: TLongintField;
    QBlob: TIBQuery;
    DSBlob: TDataSource;
    QBlobHI_TEXTE: TMemoField;
    QBlobHI_IMAGE: TBlobField;
    pmCategories: TPopupMenu;
    mTous: TMenuItem;
    mActivitesEconomiques: TMenuItem;
    mArtsEtLettres: TMenuItem;
    mCalamites: TMenuItem;
    Histoire1: TMenuItem;
    mPersonnalites: TMenuItem;
    mReligions: TMenuItem;
    mSciencesTechniques: TMenuItem;
    mVieQuotidienne: TMenuItem;
    mInfosGenealogiques: TMenuItem;
    procedure colDateDrawColumnCell(Sender: TObject;
      ACanvas: TCanvas; AViewInfo: longint; var ADone: boolean);
    procedure btnAjoutClick(Sender: TObject);
    procedure dbListeDblClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure btnExportClick(Sender: TObject);
    procedure btnImportClick(Sender: TObject);
    procedure btnHistoireClick(Sender: TObject);
    procedure cxComboBox1PropertiesChange(Sender: TObject);
    procedure doMajBouton;
    procedure ibHistoireAfterScroll(DataSet: TDataSet);
    procedure lTitreClick(Sender: TObject);
    procedure mTousClick(Sender: TObject);
  private
    { Déclarations privées }
    procedure doOpenQueries;
    function Categorie: integer;
  protected
    procedure Activate; override;
    procedure DoClose(var Action: TCloseAction); override;
    procedure Deactivate; override;
    procedure DoShow; override;
  public
    { Déclarations publiques }
    procedure Loaded; override;
    destructor Destroy; override;
    procedure doPosition(iID, iMode: integer);
  end;

implementation

uses  u_Dm,
      u_Form_Histoire_Detail,
      bgrabitmap,
      bgrabitmaptypes,
      fonctions_images,
      u_Form_Main,
      u_common_functions,
      u_common_resources,
      u_Common_Const,
      u_common_ancestro,
      u_Form_Histoire_Export,
      u_Form_Histoire_Import,
      u_common_ancestro_functions,
      u_genealogy_context,
      lazutf8classes,
      IBSQL,
      fonctions_string,
      fonctions_dialogs,
      fonctions_ini,
      LazFileUtils;

{$IFDEF FPC}
{$R *.lfm}
{$ELSE}{$R *.DFM}{$ENDIF}

procedure TFHistoire.Loaded;
begin
  f_GetMemIniFile();
  Color := gci_context.ColorLight;
  Evenement.Color := gci_context.ColorMedium;
  cxDBImage1.ShowHint := True;
  cxGrid1.ShowHint := True;
  inherited;
end;

destructor TFHistoire.Destroy;
begin
  ibHistoire.Close;
  inherited Destroy;
  _FormHistoire:=nil
end;

procedure TFHistoire.DoClose(var Action: TCloseAction);
begin
  inherited;
  Action := caFree;
end;

procedure TFHistoire.Activate;
begin
  ibHistoire.AfterScroll := ibHistoireAfterScroll;
  ibHistoireAfterScroll ( ibHistoire );
  inherited;
end;

procedure TFHistoire.Deactivate;
begin
  inherited;
  ibHistoire.AfterScroll := nil;
end;


procedure TFHistoire.DoShow;
begin
  doOpenQueries;
  inherited;
end;

procedure TFHistoire.colDateDrawColumnCell(Sender: TObject;
  ACanvas: TCanvas; AViewInfo: longint; var ADone: boolean);
begin
  acanvas.Font.Style := [fsBold];
  acanvas.Brush.Color := $00F4FFE6;
end;

procedure TFHistoire.btnAjoutClick(Sender: TObject);
var
  aFormHistoireDetail: TFormHistoireDetail;
begin
  aFormHistoireDetail := TFormHistoireDetail.Create(self);
  try
    CentreLaFiche(aFormHistoireDetail, self, 0, 0);
    aFormHistoireDetail.ShowModal;
    if aFormHistoireDetail.bModif then
    begin
      doOpenQueries;
      with aFormHistoireDetail.qHistoireHI_ID do
        if AsInteger > 0 then
          ibHistoire.locate('HI_ID', AsInteger, []);
    end;
  finally
    FreeAndNil(aFormHistoireDetail);
  end;
  SendFocus(cxGrid1);
end;

procedure TFHistoire.dbListeDblClick(Sender: TObject);
begin
  btnAjoutClick(Sender);
end;

procedure TFHistoire.btnDeleteClick(Sender: TObject);
begin
  if MyMessageDlg(rs_Youll_delete_the_record_of_this_event + _CRLF + _CRLF +
    rs_Do_you_continue, mmtWarning, [mmbYes, mmbNo], self) = mrYes then
  begin
    ibHistoire.Delete;
    btnDelete.Enabled := not ibHistoire.IsEmpty;
  end;
end;

procedure TFHistoire.btnExportClick(Sender: TObject);
var
  q: TIBQuery;
  sChaine: string;
  iVal: integer;
  B: TFileStreamUTF8;
  T: TMemoryStream;
  L: longint;
  aFHistoireExport: TFHistoireExport;
  bImport: boolean;
  iNumDossier: integer;
  sDico, sFichier: string;
begin
  aFHistoireExport := TFHistoireExport.Create(self);
  try
    aFHistoireExport.ShowModal;
    bImport := aFHistoireExport.bImport;
    iNumDossier := aFHistoireExport.iNumDossier;
    sDico := aFHistoireExport.sDico;
    sFichier := aFHistoireExport.sFichier;
  finally
    FreeAndNil(aFHistoireExport);
  end;

  if bImport then
  begin
    if length(sFichier) < 5 then
      sFichier := 'Histoire.dica';

    q := TIBQuery.Create(self);
    with q do
    try
      Database := dm.ibd_BASE;
      Transaction := dm.IBT_BASE;
      SQL.Clear;
      SQL.Add('select hi_dicorigine' + ',hi_date_texte' +
        ',hi_cat' + ',hi_titre' + ',hi_texte' + ',hi_image'
        + ' from ref_histoire' + ' where hi_dossier=' + IntToStr(iNumDossier));

      if Length(sDico) > 0 then
        SQL.Add('and hi_dicorigine=''' + sDico + '''');

      SQL.Add('order by hi_date_code_debut');
      Open;
      Last;
      First;

      if IsEmpty then
      begin
        MyShowMessage(rs_Error_Nothing_to_export);
        Close;
        exit;
      end;

      sFichier := ChangeFileExt(sFichier, '.dica');
      Screen.Cursor := crHourGlass;
      B := TFileStreamUTF8.Create(sFichier, fmCreate);
      try
        T := TMemoryStream.Create;
        try
          while not EOF do
          begin
            sChaine := '';
            sChaine := (FieldByName('hi_dicorigine').AsString);
            L := Length(sChaine);
            B.WriteBuffer(L, SizeOf(L));
            B.WriteBuffer(Pointer(sChaine)^, L);

            sChaine := '';
            sChaine := (FieldByName('hi_date_texte').AsString);

            L := Length(sChaine);
            B.WriteBuffer(L, SizeOf(L));
            B.WriteBuffer(Pointer(sChaine)^, L);

            iVal := FieldByName('hi_cat').AsInteger;
            B.WriteBuffer(iVal, sizeOf(ival));

            sChaine := '';
            sChaine := (FieldByName('hi_titre').AsString);
            L := Length(sChaine);
            B.WriteBuffer(L, SizeOf(L));
            B.WriteBuffer(Pointer(sChaine)^, L);

            T.Clear;
            TBlobField(FieldByName('hi_texte')).SaveToStream(T);
            iVal := T.Position;
            B.WriteBuffer(iVal, 4);

            if not FieldByName('hi_texte').IsNull then
              TBlobField(FieldByName('hi_texte')).SaveToStream(B);

            T.Clear;
            TBlobField(FieldByName('hi_image')).SaveToStream(T);
            iVal := T.Position;
            B.WriteBuffer(iVal, 4);

            if not FieldByName('hi_image').IsNull then
             Begin
              TBlobField(FieldByName('hi_image')).SaveToStream(B);
             end;

            Next;
          end;
          Close;
        finally
          T.Free;
        end;
      finally
        B.Free;
        Screen.Cursor := crDefault;
      end;
      MyMessageDlg(rs_History_File_has_got_this_name + _CRLF + sFichier, mmtInformation,
        [mmbOk], self);
    finally
      Free;
    end;
  end;
end;

procedure TFHistoire.btnImportClick(Sender: TObject);
var
  q: TIBQuery;
  dossier, dico, chaine: string;
  L: integer;
  B: TFileStreamUTF8;
  Tampon: TMemoryStream;
  aFHistoireImport: TFHistoireImport;
  bImport: boolean;
  sFichier: string;
  iNumDossier: integer;
  lid_Image: TBGRACustomBitmap;

  function ReadAnsiString(Stream: TFileStreamUTF8): string;
  var
    Len: integer;
  begin
    Stream.ReadBuffer(Len, 4);
    SetLength(Result, Len);
    Stream.ReadBuffer(Result[1], Len);
  end;

begin
  L := 0;

  aFHistoireImport := TFHistoireImport.Create(self);
  try
    aFHistoireImport.ShowModal;
    bImport := aFHistoireImport.bImport;
    iNumDossier := aFHistoireImport.iNumDossier;
    sFichier := aFHistoireImport.sFichier;
  finally
    FreeAndNil(aFHistoireImport);
  end;

  if bImport then
  begin
    if not FileExistsUTF8(sFichier) { *Converted from FileExistsUTF8*  } then
    begin
      MyShowMessage(fs_RemplaceMsg(rs_Error_Cannot_find_this_file,[sFichier]));
      exit;
    end;

    screen.Cursor := crHourGlass;
    try

      B := TFileStreamUTF8.Create(sFichier, fmOpenRead);
      try
        B.Seek(0, soFromBeginning);
        dossier := IntToStr(iNumDossier);
        q := TIBQuery.Create(self);
        try
          q.database := dm.ibd_BASE;
          q.Transaction := dm.IBT_BASE;

          if dossier = '0' then
          begin
            dico := ReadAnsiString(B);
            q.SQL.Add('delete from ref_histoire' +
              ' where hi_dossier = 0 and hi_dicorigine=''' + dico + '''');
            q.ExecSQL;
          end;

          q.SQL.Clear;
          q.SQL.Add('insert into ref_histoire (' + 'hi_dossier'
            + ',hi_dicorigine' + ',hi_date_texte' +
            ',hi_cat' + ',hi_titre' + ',hi_texte' +
            ',hi_image' + ')' + ' values(' + dossier
            + ',:hi_dicorigine' + ',:hi_date_texte' +
            ',:hi_cat' + ',:hi_titre' + ',:hi_texte' +
            ',:hi_image' + ')');

          Tampon := TMemoryStream.Create;
          lid_Image:=fci_GetCustomImage;
          try
            B.Seek(0, soFromBeginning);
            with q, Params do
            while B.Position < B.Size do
            begin
              chaine := '';
              chaine := ReadAnsiString(B);
              ParamByName( IBQ_hi_dicorigine ).AsString := chaine;

              chaine := '';
              chaine := ReadAnsiString(B);
              ParamByName( IBQ_hi_date_texte ).AsString := chaine;

              B.ReadBuffer(L, 4);

              ParamByName( IBQ_hi_cat ).AsInteger := L;

              chaine := '';
              chaine := ReadAnsiString(B);
              ParamByName( IBQ_hi_titre ).AsString := chaine;

              B.ReadBuffer(L, 4);

              if L = 0 then
                ParamByName( IBQ_hi_texte ).Value := Null
              else
              begin
                tampon.Clear;
                tampon.CopyFrom(B, L);
                tampon.Seek(0, soFromBeginning);
                ParamByName( IBQ_hi_texte ).LoadFromStream(tampon, ftMemo);
              end;

              B.ReadBuffer(L, 4);

              if L = 0 then
                ParamByName( IBQ_hi_image ).Value := Null
              else
              begin
                tampon.Clear;
                tampon.CopyFrom(B, L);
                tampon.Seek(0, soFromBeginning);
                p_StreamToCustomImage ( tampon, lid_Image );
                p_ImageToStream ( lid_Image, tampon );
                ParamByName( IBQ_hi_image ).LoadFromStream(tampon, ftBlob);
              end;
              ExecSQL;
            end;
            tampon.Clear;
          finally
            tampon.Destroy;
            lid_Image.Destroy;
          end;
          q.Close;
        finally
          q.Free;
        end;
      finally
        B.Free;
      end;
      doOpenQueries;
    finally
      screen.Cursor := crDefault;
    end;
    MyMessageDlg(fs_RemplaceMsg(rs_This_file_has_been_imported_with_success, [sFichier]), mmtInformation,
      [mmbOk], self);
  end;
end;

procedure TFHistoire.doPosition(iID, iMode: integer);
var
  q: TIBSQL;
  ilID: integer;
begin
  q := TIBSQL.Create(Application);
  with q, SQL do
  try
    DataBase := dm.ibd_BASE;
    Transaction := dm.IBT_BASE;
    if iMode = 0 then
      Text := 'select first(1)hi_id from ref_histoire h' +
        ',(select ev_ind_date_year as ev_year' + ',ev_ind_date_mois as ev_mois'
        + ',ev_ind_date_jour as ev_jour ' +
          ',ev_ind_calendrier1 as ev_cal ' +
        'from evenements_ind where ev_ind_clef=:evenement)e'
    else
      Text := 'select first(1)hi_id from ref_histoire h' +
        ',(select ev_fam_date_year as ev_year' + ',ev_fam_date_mois as ev_mois'
        + ',ev_fam_date_jour as ev_jour ' +
          ',ev_fam_calendrier1 as ev_cal ' +
        'from evenements_fam where ev_fam_clef=:evenement)';

    Add('where h.hi_dossier in (0,' + IntToStr(dm.NumDossier) +
      ')' + 'and h.hi_date_code_debut<=(select date_code from proc_date_code(ev_year,ev_mois,ev_jour,2,ev_cal))');
    if Categorie <> 0 then
      Add('and HI_CAT=' + IntToStr(Categorie));
    Add('order by hi_date_code_debut desc');
    Params[0].AsInteger := iID;
    try
      ExecQuery;
      if not Eof then
        ilID := Fields[0].AsInteger;
    except
      ilID := 0;
    end;
    Close;
  finally
    Destroy;
  end;

  with ibHistoire do
   Begin
    if not Active then
      doOpenQueries;
    DisableControls;
    try
      if ilID > 0 then
        locate('HI_ID', ilID, []);
    finally
      EnableControls;
    end;
   end;
  doMajBouton;
  if Active and Visible then
    cxGrid1.SetFocus;
end;

procedure TFHistoire.doOpenQueries;
begin
  ibHistoire.DisableControls;
  with ibHistoire do
    try
      QBlob.DisableControls;
      QBlob.Close;
      Close;
      SQL.Text := 'select hi_id,hi_dossier,hi_dicorigine,hi_date_texte,hi_cat,hi_titre'
        + ',hi_date_code_debut' + ' from ref_histoire where hi_dossier in (0,' +
        IntToStr(dm.NumDossier) + ')';

      if Categorie <> 0 then
        SQL.Add('and hi_cat=' + IntToStr(Categorie));

      SQL.Add('order by hi_date_code_debut');
      Open;
    finally
      QBlob.EnableControls;
      ibHistoire.EnableControls;
    end;

  case ibHistoire.RecordCount of
    0: Caption := rs_Caption_History_No_event;
    1: Caption := rs_Caption_History_One_event;
    else
      Caption := fs_RemplaceMsg(rs_Caption_History_Events,
        [IntToStr(ibHistoire.RecordCount)]);
  end;
  cxLabel2.Visible := IBHistoire.RecordCount <= 0;
  if Categorie = 0 then
  begin
    cxLabel2.Font.Style := [fsBold];
    cxLabel2.Caption := rs_Caption_History_module_is_empty_Clic_on_Import_button;
  end
  else
  begin
    cxLabel2.Font.Style := [];
    cxLabel2.Caption := rs_Caption_History_No_event_in_folder_change_folder_or_import;
  end;
  with ibHistoire do
    Begin
      btnExport.Enabled := not IsEmpty;
      btnHistoire.Enabled := not IsEmpty;
      btnDelete.Enabled := not IsEmpty;
    end;
end;

procedure TFHistoire.btnHistoireClick(Sender: TObject);
var
  an, mois, jour: integer;
begin
  DecodeDateCode(ibHistoireHI_DATE_CODE_DEBUT.AsInteger, an, mois, jour,cGRE);
  GoToThisUrl(rs_URL_WIKIPEDIA + IntToStr(an));
end;

procedure TFHistoire.cxComboBox1PropertiesChange(Sender: TObject);
begin
  doOpenQueries;
end;

procedure TFHistoire.doMajBouton;
var
  an, mois, jour: integer;
begin
  DecodeDateCode(ibHistoireHI_DATE_CODE_DEBUT.AsInteger, an, mois, jour,cGRE);
  btnHistoire.Hint := rs_Hint_Story_of_year + IntToStr(an);
  with QBlob do
    try
      DisableControls;
      Close;
      Params [0].AsInteger := ibHistoirehi_id.AsInteger;
      Open;

    finally
      EnableControls;
    end;
end;

procedure TFHistoire.ibHistoireAfterScroll(DataSet: TDataSet);
begin
  doMajBouton;
  if cxDBImage1.Picture.Bitmap.Handle > 0 Then
  with qblob do
    Begin
      DisableControls;
      try
        Edit;
        Post;
      finally
        EnableControls;
      end;
    end;
end;

procedure TFHistoire.lTitreClick(Sender: TObject);
begin

end;

function TFHistoire.Categorie: integer;
var
  i: integer;
begin
  Result := -1;
  for i := 0 to pmCategories.Items.Count - 1 do
    if pmCategories.Items[i].Checked then
    begin
      Result := i;
      Break;
    end;
end;

procedure TFHistoire.mTousClick(Sender: TObject);
var
  i: integer;
begin
  btnCategorie.Hint := rs_Hint_Events_folder + (Sender as TMenuItem).Caption;
  for i := 0 to pmCategories.Items.Count - 1 do
    pmCategories.Items[i].Checked := (pmCategories.Items[i] = (Sender as TMenuItem));
  doOpenQueries;
end;

end.


















