{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_Form_Indi_DateIncoherentes;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  jpeg, TPanelUnit, Windows,
{$ELSE}
  LCLIntf,
{$ENDIF}
  U_FormAdapt,Forms,Variants,Graphics,Dialogs,Menus,DB,IBQuery,
  StdCtrls,Controls,ExtCtrls,
  u_reports_components,
  Classes,u_buttons_appli,
  PrintersDlgs, u_ancestropictimages,
  U_ExtDBGrid, Grids, DBGrids;

type

  { TFIndiDateIncoherente }

  TFIndiDateIncoherente=class(TF_FormAdapt)
    pBorder:TPanel;
    pGeneral:TPanel;
    pGlobal:TPanel;
    pBottom:TPanel;
    IBIncoherenreces:TIBQuery;
    DSIncoherenreces:TDataSource;
    pmUnions:TPopupMenu;
    Panel3:TPanel;
    Image1:TIATitle;
    Panel2:TPanel;
    ExporterenHTML1:TMenuItem;
    SaveDialog:TSaveDialog;
    Label1:TLabel;
    dxComponentPrinter1:TPrinterSetupDialog;
    N1:TMenuItem;
    mOuvreFiche:TMenuItem;
    mImprime:TMenuItem;
    dxDBGrid1:TExtDBGrid;
    fpBoutons: TPanel;
    GoodBtn7: TFWClose;
    btnPrint: TFWPrintGrid;
    rbFormat: TRadioButton;
    rbValeur: TRadioButton;

    procedure dxDBGrid1DrawColumnCell(Sender: TObject; const Rect: TRect;
      DataCol: Integer; Column: TColumn; State: TGridDrawState);
    procedure ExporterenHTML1Click(Sender:TObject);
    procedure pmUnionsPopup(Sender:TObject);
    procedure btnPrintClick(Sender:TObject);
    procedure mOuvreFicheClick(Sender:TObject);
    procedure mImprimeClick(Sender:TObject);
    procedure dxDBGrid1DBTableView1CellDblClick(
      Sender:TObject;
      ACellViewInfo:Longint;AButton:TMouseButton;
      AShift:TShiftState;var AHandled:Boolean);
    procedure rbFormatClick(Sender: TObject);

  private
    fCleFicheSelected:integer;
    bFirstOpen:boolean;
    procedure doOpen;
  protected
    procedure DoClose(var Action: TCloseAction); override;
  public
    procedure Loaded; override;
    destructor Destroy; override;

    property CleFicheSelected:integer read fCleFicheSelected write fCleFicheSelected;
    procedure FirstOpen;

  end;

implementation

uses u_dm,
     u_common_const,
     u_common_resources,
     u_common_ancestro,
     u_genealogy_context,
     fonctions_dialogs,
     fonctions_ini,
     u_form_main,
     u_common_ancestro_functions,
     fonctions_components;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}

procedure TFIndiDateIncoherente.Loaded;
begin
  f_GetMemIniFile();
  Color:=gci_context.ColorLight;
  // Matthieu ?
  // dxDBGrid1DBTableView1.RestoreFromRegistry(gci_context.KeyRegistry+DirectorySeparator+'W_INDIS_DATEINCOHERENTES');
  SaveDialog.InitialDir:=gci_context.PathDocs;
  inherited;
end;

destructor TFIndiDateIncoherente.Destroy;
begin
  inherited Destroy;
  FMain.aFIndiDateIncoherente:=nil;
end;

procedure TFIndiDateIncoherente.dxDBGrid1DrawColumnCell(Sender: TObject;
  const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState);
var
  value,valueSexe:Variant;
begin
  with dxDBGrid1,Datasource.Dataset do
    Begin
      // Matthieu : pas par colonne
      Value:=FieldByName('TITRE').Value;
      valueSexe:=FieldByName('Sexe').Value;

      if not VarIsNull(Value) then
      begin
        if (Value=1) then
          Canvas.Font.Style:= [fsBold]
        else
          Canvas.Font.Style:= [];
      end;

      if not VarIsNull(valueSexe) then
      begin
        if (valueSexe=1) then
          Canvas.Font.Color:=gci_context.ColorHomme
        else
          Canvas.Font.Color:=gci_context.ColorFemme;
      end;
      if Column.Field.Value='X' then
        Canvas.Font.Style:=[fsBold];
      if Column = SelectedColumn then
        if Canvas.Brush.Color=Color then
          Canvas.Brush.Color:=gci_context.ColorMedium;
    end;
end;

procedure TFIndiDateIncoherente.DoClose(
  var Action:TCloseAction);
begin
  IBIncoherenreces.Close;
  // Matthieu ?
  //dxDBGrid1DBTableView1.StoreToRegistry(gci_context.KeyRegistry+DirectorySeparator+'W_INDIS_DATEINCOHERENTES');
  Action:=caFree;
  inherited;
end;

procedure TFIndiDateIncoherente.ExporterenHTML1Click(Sender:TObject);
var
  SavePlace:TBookmark;
begin
  SavePlace:=nil;
  SaveDialog.FileName:='Liste des dates incoherentes.HTM';
  if SaveDialog.Execute then
  begin
    IBIncoherenreces.DisableControls;
    try
      SavePlace:=IBIncoherenreces.GetBookmark;
      ExportGridToHTML(SaveDialog.FileName,dxDBGrid1,True,True);
    finally
      IBIncoherenreces.GotoBookmark(SavePlace);
      IBIncoherenreces.FreeBookmark(SavePlace);
      IBIncoherenreces.EnableControls;
    end;
  end;
end;

procedure TFIndiDateIncoherente.pmUnionsPopup(Sender:TObject);
begin
  ExporterenHTML1.Enabled:=(IBIncoherenreces.Active)and(IBIncoherenreces.IsEmpty=false);
end;


procedure TFIndiDateIncoherente.doOpen;
var
  i:integer;
begin
  // Matthieu ?
//  dxDBGrid1.OptionsView.NoDataToDisplayInfoText:='Recherche en cours';
  dxDBGrid1.SetFocus;
  screen.Cursor:=crHourGlass;
  doShowWorking(rs_Wait+_CRLF+rs_Wait_Asking_to_database);
  
  try
    with IBIncoherenreces, Params do
    begin
      Close;
      for i:=0 to Params.Count-1 do
        Params[i].AsInteger:=0;
      ParamByName( IBQ_I_DOSSIER ).AsInteger:=dm.NumDossier;
      if gci_context.CheckAgeMiniMariage then
      begin
        ParamByName( IBQ_MIN_MAR_HOM ).AsInteger:=gci_context.AgeMiniMariageHommes;
        ParamByName( IBQ_MIN_MAR_FEM ).AsInteger:=gci_context.AgeMiniMariageFemmes;
      end;
      if gci_context.CheckAgeMaxiMariage then
      begin
        ParamByName( IBQ_MAX_MAR_HOM ).AsInteger:=gci_context.AgeMaxiMariageHommes;
        ParamByName( IBQ_MAX_MAR_FEM ).AsInteger:=gci_context.AgeMaxiMariageFemmes;
      end;
      if gci_context.CheckAgeMiniNaissanceEnfant then
      begin
        ParamByName( IBQ_MIN_ENF_HOM ).AsInteger:=gci_context.AgeMiniNaissanceEnfantHommes;
        ParamByName( IBQ_MIN_ENF_FEM ).AsInteger:=gci_context.AgeMiniNaissanceEnfantFemmes;
      end;
      if gci_context.CheckAgeMaxiNaissanceEnfant then
      begin
        ParamByName( IBQ_MAX_ENF_HOM ).AsInteger:=gci_context.AgeMaxiNaissanceEnfantHommes;
        ParamByName( IBQ_MAX_ENF_FEM ).AsInteger:=gci_context.AgeMaxiNaissanceEnfantFemmes;
      end;
      if gci_context.CheckEsperanceVie then
      begin
        ParamByName( IBQ_MAX_VIE_HOM ).AsInteger:=gci_context.AgeMaxiAuDecesHommes;
        ParamByName( IBQ_MAX_VIE_FEM ).AsInteger:=gci_context.AgeMaxiAuDecesFemmes;
      end;
      if gci_context.CheckEcartAgeEpoux
        then ParamByName( IBQ_MAX_ECART_EPOUX ).AsInteger:=gci_context.EcartAgeEntreEpoux;
      if gci_context.CheckNbJourEntreNaissance then
      begin
        ParamByName( IBQ_MIN_ENTRE_ENF ).AsInteger:=gci_context.NbJourEntre2NaissancesNormales;
        ParamByName( IBQ_MAX_ENTRE_JUMEAUX ).AsInteger:=gci_context.NbJourEntre2NaissancesJumeaux;
      end;
      if rbFormat.Checked then
        ParamByName( IBQ_MODE ).AsInteger:=2
      else
        ParamByName( IBQ_MODE ).AsInteger:=1;

      Open;
    end;
  except
    MyShowMessage(rs_Error_Cannot_connect_to_database_or_dates_not_compatible);
  end;
  doCloseWorking;
//  dxDBGrid1DBTableView1.OptionsView.NoDataToDisplayInfoText:='Pas de résultats';
  screen.Cursor:=crDefault;
end;

procedure TFIndiDateIncoherente.btnPrintClick(Sender:TObject);
begin
  btnPrint.DBtitle := Label1.Caption;
end;

procedure TFIndiDateIncoherente.mOuvreFicheClick(Sender:TObject);
begin
  if (IBIncoherenreces.Active)and(not IBIncoherenreces.IsEmpty) then
  begin
    // Matthieu ?
    if dxDBGrid1.DataSource.DataSet.FieldByName('CLE_FICHE').AsString>'' then
    begin
      fCleFicheSelected:=IBIncoherenreces.FieldByName('clef_ind').AsInteger;
      if fCleFicheSelected>0 then
      begin
        dm.individu_clef:=fCleFicheSelected;
        FMain.Ouvre_Et_Selecte_Individu;
      end;
    end;
  end;
end;

procedure TFIndiDateIncoherente.mImprimeClick(Sender:TObject);
begin
  btnPrintClick(Sender);
end;

procedure TFIndiDateIncoherente.dxDBGrid1DBTableView1CellDblClick(
  Sender:TObject;
  ACellViewInfo:Longint;AButton:TMouseButton;
  AShift:TShiftState;var AHandled:Boolean);
begin
  mOuvreFicheClick(Sender)
end;

procedure TFIndiDateIncoherente.FirstOpen;
var
  reponse:word;
begin
  Enabled:=false;
  reponse:=MyMessageDlg(rs_Do_you_want_to_see_bad_formated_dates,mmtConfirmation
    ,[mmbYes,mmbNo,mmbCancel],Self);
  Enabled:=true;
  bFirstOpen:=true;
  if reponse=mrYes then
    rbFormat.Checked:=true
  else if reponse=mrNo then
    rbValeur.Checked:=true;
  
  bFirstOpen:=false;
  if reponse<>mrCancel then
    doOpen
//  Matthieu ?
  //else
    //dxDBGrid1DBTableView1.OptionsView.NoDataToDisplayInfoText:='Pas de résultats';
end;

procedure TFIndiDateIncoherente.rbFormatClick(Sender: TObject);
begin
  if not bFirstOpen then
    doOpen;
  btnPrint.DestroyReport;
end;

end.

