{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_Form_Liste_Doublons;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  jpeg, TPanelUnit, Windows,
{$ELSE}
{$ENDIF}
  U_FormAdapt, U_OnFormInfoIni, Forms, Dialogs, IB, Menus, 
  DB, IBQuery, StdCtrls, Controls, ExtCtrls, U_ExtDBGrid, Classes,
  u_buttons_appli, ExtJvXPCheckCtrls, PrintersDlgs, u_ancestropictimages, IBSQL,
  Grids, DBGrids;

type

  { TFListeDoublons }

  TFListeDoublons=class(TF_FormAdapt)
    OnFormInfoIni: TOnFormInfoIni;
    pBorder:TPanel;
    pGeneral:TPanel;
    pGlobal:TPanel;
    pBottom:TPanel;
    IBQDoublons:TIBQuery;
    DSDoublons:TDataSource;
    dxDBGrid1:TExtDBGrid;
    Panel3:TPanel;
    Image1:TIATitle;
    Panel2:TPanel;
{$IFNDEF FPC}
    IBQDoublonsCLE_FICHE:TLongintField;
    IBQDoublonsNOM:TIBStringField;
    IBQDoublonsPRENOM:TIBStringField;
    IBQDoublonsDATE_NAISS:TIBStringField;
    IBQDoublonsLIEU_NAISS:TIBStringField;
    IBQDoublonsDATE_DECES:TIBStringField;
    IBQDoublonsLIEU_DECES:TIBStringField;
{$ENDIF}
    pBravo:TPanel;
    Image2:TImage;
    Label1:TLabel;
    Label2:TLabel;
    
    pmGrid:TPopupMenu;
    mOuvrirlafiche:TMenuItem;
    mSupprimerlafiche:TMenuItem;
    N1:TMenuItem;
    dxComponentPrinter1:TPrinterSetupDialog;
    SaveDialog:TSaveDialog;
    IBSQLDeleteIndi:TIBSQL;
    fpBoutons:TPanel;
    btnFermer:TFWClose;
    cxButton1:TFWSearch;
    cbElargir:TJvXPCheckBox;
    btnPrint:TFWPrint;
    btnExport:TFWExport;
    mConsulterlafiche:TMenuItem;
    cbIndiEnCours: TJvXPCheckBox;

    procedure dxDBGrid1DrawColumnCell(Sender: TObject; const Rect: TRect;
      DataCol: Integer; Column: TColumn; State: TGridDrawState);
    procedure pmGridPopup(Sender:TObject);
    procedure mOuvrirlaficheClick(Sender:TObject);
    procedure mSupprimerlaficheClick(Sender:TObject);
    procedure cxButton1Click(Sender:TObject);
    procedure btnPrintClick(Sender:TObject);
    procedure btnExportClick(Sender:TObject);
    procedure dxDBGrid1DBTableView1CellDblClick(
      Sender:TObject;
      ACellViewInfo:Longint;AButton:TMouseButton;
      AShift:TShiftState;var AHandled:Boolean);
    procedure mConsulterlaficheClick(Sender:TObject);
  private
    fDialogMode:boolean;

    procedure SetDialogMode(const Value:boolean);
    procedure doShowDoublons;
  protected
    procedure DoClose(var Action: TCloseAction); override;
  public
    procedure Loaded; override;
    destructor Destroy; override;
    procedure DoRefreshControls; override;
    procedure DoShowFirstTime; override;
    property DialogMode:boolean read fDialogMode write SetDialogMode;
  end;

implementation

uses u_dm, u_form_main,
     u_common_const,
     u_common_resources,
     fonctions_dialogs,
     u_common_functions,
     u_common_ancestro,math,
     u_genealogy_context,
     u_common_ancestro_functions,
     fonctions_ini,
     fonctions_components;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}

procedure TFListeDoublons.Loaded;
begin
  f_GetMemIniFile();
  Color:=gci_context.ColorLight;
  pBravo.Color:=gci_context.ColorLight;
  Height:=round(TControl(Owner).height*gci_context.TailleFenetre/100);
  width:=max(round(TControl(Owner).width*gci_context.TailleFenetre/100),729);

  cbElargir.Hint:=rs_Hint_Unless_identity_of_names_Duplicates_will_be_searched;
  cbIndiEnCours.Hint:=rs_Hint_Duplicates_will_be_searched_in_persons_with_a_same_or_similar_surname;
  inherited;
end;

destructor TFListeDoublons.Destroy;
begin
  inherited Destroy;
  FMain.aFListeDoublons:=nil;
end;

procedure TFListeDoublons.dxDBGrid1DrawColumnCell(Sender: TObject;
  const Rect: TRect; DataCol: Integer; Column: TColumn; State: TGridDrawState);
begin
  with dxDBGrid1 do
  if  Column = SelectedColumn then
    if Canvas.Brush.Color=Color then
      Canvas.Brush.Color:=gci_context.ColorMedium;
end;

procedure TFListeDoublons.doShowDoublons;
var
  fTryToSelectCleFiche:integer;
begin
  screen.Cursor:=crHourglass;
  
  fTryToSelectCleFiche:=-1;
  if IBQDoublons.Active then
    if not IBQDoublons.IsEmpty then
      fTryToSelectCleFiche:=IBQDoublons.FieldByName('CLE_FICHE').AsInteger;

  with IBQDoublons do
  try
    Close;
    Params[0].AsInteger:=dm.NumDossier;
    if cbIndiEnCours.Checked then
      Params[1].AsInteger:=dm.individu_clef
    else
      Params[1].AsInteger:=0;

    if cbElargir.Checked then
      Params[2].AsInteger:=1
    else
      Params[2].AsInteger:=0;

    Open;

    btnPrint.Enabled:=not dxDBGrid1.DataSource.Dataset.IsEmpty;
    btnExport.Enabled:=btnPrint.Enabled;
  except
    on E:EIBError do
    begin
      MyShowMessage(rs_Error_duplication_request_problem+_CRLF+_CRLF+
        E.Message);
    end;
  end;

  if fTryToSelectCleFiche<>-1 then
    IBQDoublons.Locate('CLE_FICHE',fTryToSelectCleFiche, []);

  doRefreshControls;
  screen.Cursor:=crDefault;
end;

procedure TFListeDoublons.DoClose(
  var Action:TCloseAction);
begin
  IBQDoublons.Close;
  dm.IBT_base.CommitRetaining;
  Action:=caFree;//obligatoire si on veut qu'une SuperForm se libère seule (elle ignore AutoFreeOnClose)
  inherited;
end;

procedure TFListeDoublons.SetDialogMode(const Value:boolean);
begin
  fDialogMode:=Value;
end;

procedure TFListeDoublons.pmGridPopup(Sender:TObject);
var
  ok:boolean;
begin
  ok:=(IBQDoublons.Active)and(not IBQDoublons.IsEmpty)and(dm.individu_clef<>IBQDoublons.FieldByName('CLE_FICHE').AsInteger);
  mOuvrirlafiche.enabled:=ok;
  mConsulterlafiche.enabled:=ok;
  mSupprimerlafiche.enabled:=ok;
end;

procedure TFListeDoublons.mOuvrirlaficheClick(Sender:TObject);
begin
  if (IBQDoublons.Active)and(not IBQDoublons.IsEmpty)
    and(dm.individu_clef<>IBQDoublons.FieldByName('CLE_FICHE').AsInteger) then
  begin
    dm.individu_clef:=IBQDoublons.FieldByName('CLE_FICHE').AsInteger;
    FMain.Ouvre_Et_Selecte_Individu;
  end;
end;

procedure TFListeDoublons.mSupprimerlaficheClick(Sender:TObject);
begin
  if MyMessageDlg(rs_Confirm_deleting_selected_person_file,mmtWarning, mmbYesNo,Self)=mrYes then
  begin
    try
      IBSQLDeleteIndi.Params[0].AsInteger:=IBQDoublons.FieldByName('CLE_FICHE').AsInteger;
      IBSQLDeleteIndi.ExecQuery;
      IBSQLDeleteIndi.Close;

      dm.IBT_base.CommitRetaining;

      doShowDoublons;
    except
    end;
  end;

end;

procedure TFListeDoublons.DoRefreshControls;
begin
  dxDBGrid1.Enabled:=(IBQDoublons.Active)and(IBQDoublons.IsEmpty=false);
  pBravo.Visible:=(IBQDoublons.Active)and(IBQDoublons.IsEmpty);
end;

procedure TFListeDoublons.cxButton1Click(Sender:TObject);
begin
  doShowDoublons;
end;

procedure TFListeDoublons.btnPrintClick(Sender:TObject);
begin
// Matthieu Après
  //dxComponentPrinter1.Preview(True,nil);
end;

procedure TFListeDoublons.btnExportClick(Sender:TObject);
var
  SavePlace:TBookmark;
  s_Fichier:string;
begin
  SavePlace:=nil;
  s_Fichier:='Liste des doublons.HTM';
  SaveDialog.FileName:=s_Fichier;
  SaveDialog.InitialDir:=gci_context.PathDocs;

  if SaveDialog.Execute then
  begin
    IBQDoublons.DisableControls;
    try
      SavePlace:=IBQDoublons.GetBookmark;

      ExportGridToHTML(SaveDialog.FileName,dxDBGrid1,True,True);
    finally
      IBQDoublons.GotoBookmark(SavePlace);
      IBQDoublons.FreeBookmark(SavePlace);

      IBQDoublons.EnableControls;

    end;
  end;
end;

procedure TFListeDoublons.dxDBGrid1DBTableView1CellDblClick(
  Sender:TObject;
  ACellViewInfo:Longint;AButton:TMouseButton;
  AShift:TShiftState;var AHandled:Boolean);
begin
  mOuvrirlaficheClick(Sender);
end;

procedure TFListeDoublons.mConsulterlaficheClick(Sender:TObject);
begin
  OpenFicheIndividuInBox(IBQDoublons.FieldByName('CLE_FICHE').AsInteger);
end;

procedure TFListeDoublons.DoShowFirstTime;
begin
  Caption:=rs_Caption_List_of_probably_cloned_persons;
end;

end.

