{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{ André Langlet 2011: il n'y a que le nom de l'unité qui n'a pas changé }
{-----------------------------------------------------------------------}

unit u_form_TAG_Gedcom;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType,
{$ENDIF}
  U_FormAdapt,DB,Forms,IBQuery,StdCtrls,U_ExtDBGrid,Controls,Classes,ExtCtrls,IBUpdateSQL,
  u_buttons_appli;

type
  TFTAGGedcom=class(TF_FormAdapt)
    Query:TIBQuery;
    DataSource:TDataSource;
    pBorder:TPanel;
    Panel7:TPanel;
    Panel4:TPanel;
    lbTitle:TLabel;
    Panel5:TPanel;
    Panel6:TPanel;
    Panel1:TPanel;
    dxDBGrid1:TExtDBGrid;
    IBUpdateSQL1:TIBUpdateSQL;
    btnFermer:TFWClose;
    btnImport:TFWImport;
    btnExport:TFWExport;
    QueryREF_EVE_LIB_COURT: TStringField;
    QueryREF_EVE_LIB_LONG: TStringField;
    QueryREF_EVE_VISIBLE: TLongintField;
    QueryREF_EVE_A_TRAITER: TLongintField;
    QueryREF_EVE_LANGUE: TStringField;
    QueryREF_EVE_OBLIGATOIRE: TLongintField;
    procedure btnFermerClick(Sender:TObject);
    procedure btnImportClick(Sender:TObject);
    procedure btnExportClick(Sender:TObject);
    procedure QueryBeforePost(DataSet: TDataSet);
    procedure DataSourceDataChange(Sender: TObject; Field: TField);
  private
    procedure OpenQuery;
  protected
    procedure DoClose(var Action: TCloseAction); override;
    procedure ControlKeyDown(var Key:Word;
      Shift:TShiftState); override;
  public
    procedure DoShowFirstTime; override;

    procedure Loaded; override;

  end;

implementation

uses u_dm,
     fonctions_db,
     fonctions_dialogs,
     u_form_export_table_2_textfile,
     u_form_Import_textfile_2_table,
     u_common_functions,
     u_common_ancestro,
     u_genealogy_context,
     fonctions_ini,
     u_common_ancestro_functions,
     SysUtils,Dialogs;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}

{ TFTAGGedcom }

procedure TFTAGGedcom.OpenQuery;
begin
  Query.DisableControls;
  try
    Query.Close;
    Query.Params[0].AsString:=gci_context.Langue;
    Query.Open;
  finally
    Query.EnableControls;
  end;
end;

procedure TFTAGGedcom.DoShowFirstTime;
begin
  Caption:=rs_Caption_Labels_of_default_events;
  OpenQuery;
end;

procedure TFTAGGedcom.ControlKeyDown(
  var Key:Word;Shift:TShiftState);
begin
  inherited;
  if Key=VK_ESCAPE then
  begin
    dm.IBTrans_Secondaire.RollbackRetaining;
    OpenQuery;
  end;
end;

procedure TFTAGGedcom.Loaded;
begin
  f_GetMemIniFile();
  Panel7.Color:=gci_context.ColorLight;
  inherited;
end;

procedure TFTAGGedcom.btnFermerClick(Sender:TObject);
begin
  p_DatasetPostIfEdited(Query);
end;

procedure TFTAGGedcom.DoClose(
  var Action:TCloseAction);
begin
  Query.Close;
  dm.IBTrans_Secondaire.CommitRetaining;
end;

procedure TFTAGGedcom.btnImportClick(Sender:TObject);
var
  aFImportTextFile2Table:TFImportTextFile2Table;
begin
  aFImportTextFile2Table:=TFImportTextFile2Table.create(self);
  try
    aFImportTextFile2Table.TableName:='REF_EVENEMENTS';
    aFImportTextFile2Table.NamePrimaryIndex:='REF_EVE_CODE';
    aFImportTextFile2Table.NameGeneratorPrimaryIndex:='GEN_REF_EVENEMENTS';
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_CODE');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_LIB_COURT');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_LIB_LONG');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_CAT');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_VISIBLE');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_A_TRAITER');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_ECRAN');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_OBLIGATOIRE');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_LANGUE');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_TYPE');
    aFImportTextFile2Table.ParamsTable.Add('REF_EVE_UNE_FOIS');
    aFImportTextFile2Table.chbVider.Checked:=true;
    if aFImportTextFile2Table.ShowModal=mrOk then
      OpenQuery;
  finally
    aFImportTextFile2Table.free;
  end;
end;

procedure TFTAGGedcom.btnExportClick(Sender:TObject);
var
  aFExportTable2TextFile:TFExportTable2TextFile;
begin
  aFExportTable2TextFile:=TFExportTable2TextFile.create(self);
  try
    aFExportTable2TextFile.TableName:='REF_EVENEMENTS';
    aFExportTable2TextFile.PropositionFileNameExport:='Tags_Gedcom.txt';
    aFExportTable2TextFile.ShowModal;
  finally
    aFExportTable2TextFile.Free;
  end;
end;

procedure TFTAGGedcom.QueryBeforePost(DataSet: TDataSet);
begin
  if Trim(DataSet.FieldByName('REF_EVE_LIB_LONG').AsString)='' then
  begin
    MyMessageDlg(rs_Error_label_not_to_be_empty_or_spaces_filled
      ,mmtError, [mmbOk],Self);
    SendFocus(dxDBGrid1);
    Abort;
  end;
end;

procedure TFTAGGedcom.DataSourceDataChange(Sender: TObject; Field: TField);
begin
  if QueryREF_EVE_OBLIGATOIRE.AsInteger=1 then
  begin
    if QueryREF_EVE_A_TRAITER.AsInteger<>1 then
    begin
      p_DatasetEditIfNotEdited(Query);
      QueryREF_EVE_A_TRAITER.AsInteger:=1;
    end;
  end;
end;

end.

