{-----------------------------------------------------------------------}
{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{ Ancestromania                                                         }
{ Création u_form_edit_ref_Divers André Langlet 2009                 }
{-----------------------------------------------------------------------}

unit u_form_edit_ref_Divers;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType,
{$ENDIF}
  U_FormAdapt,Forms,
  DB,U_ExtDBGrid,
  StdCtrls,Controls,Classes,ExtCtrls,
  u_buttons_appli,
  IBTable;

type
  TFEditRefDivers=class(TF_FormAdapt)
    DataSource:TDataSource;
    pBorder:TPanel;
    Panel7:TPanel;
    Panel4:TPanel;
    lbTitle:TLabel;
    Panel8:TPanel;
    Panel6:TPanel;
    Panel1:TPanel;
    dxDBGrid1:TExtDBGrid;
    btnAdd:TFWAdd;
    btnDel:TFWDelete;
    btnImport:TFWImport;
    btnExport:TFWExport;
    btnFermer:TFWClose;
    Query:TIBTable;
{$IFNDEF FPC}
    QueryREF_LANGUE:TStringField;
    QueryREF_LIBELLE:TStringField;
{$ENDIF}
    procedure btnAddClick(Sender:TObject);
    procedure btnDelClick(Sender:TObject);
    procedure btnExportClick(Sender:TObject);
    procedure btnImportClick(Sender:TObject);
    procedure QueryAfterInsert(DataSet:TDataSet);
    procedure QueryBeforeDelete(DataSet:TDataSet);
    procedure QueryAfterDelete(DataSet:TDataSet);
    procedure QueryBeforePost(DataSet:TDataSet);
  private
    procedure OpenQuery;
    procedure DoClose(var Action: TCloseAction); override;
    procedure ControlKeyDown(var Key:Word;
      Shift:TShiftState); override;
  public
    NomTable:string;
    procedure Loaded; override;
    procedure DoShowFirstTime; override;
    procedure DoRefreshControls; override;
  end;

implementation

uses  u_dm,
      u_common_functions,u_common_ancestro,
      u_common_ancestro_functions,
      u_form_export_table_2_textfile,
      u_form_Import_textfile_2_table,
      u_genealogy_context,
      fonctions_dialogs,
      fonctions_ini,
      Dialogs,
      SysUtils;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}

{ TFEditRefDivers }

procedure TFEditRefDivers.Loaded;
begin
  f_GetMemIniFile();
  Panel7.Color:=gci_context.ColorLight;
  btnAdd.Color:=gci_context.ColorLight;
  btnDel.Color:=gci_context.ColorLight;
  btnAdd.ColorFrameFocus:=gci_context.ColorLight;
  btnDel.ColorFrameFocus:=gci_context.ColorLight;
  inherited;
end;

procedure TFEditRefDivers.DoShowFirstTime;
begin
  Query.TableName:=NomTable;
  Width:=310;
  if NomTable='REF_DIVERS_IND' then
    Caption:=rs_Caption_Label_of_other_events_of_persons
  else if NomTable='REF_DIVERS_FAM' then
    Caption:=rs_Caption_Label_of_other_events_of_families
  else if NomTable='REF_MARR' then
  begin
    Width:=360;
    Caption:=rs_Caption_Label_of_unions;
  end;
  OpenQuery;
end;

procedure TFEditRefDivers.OpenQuery;
begin
  Query.DisableControls;
  try
    Query.Close;
    Query.Filtered:=false;
    Query.Open;
    Query.Filter:='REF_LANGUE='''+gci_context.Langue+'''';
    Query.Filtered:=true;
  finally
    Query.EnableControls;
  end;
  doRefreshControls;
end;

procedure TFEditRefDivers.DoRefreshControls;
begin
  btnAdd.enabled:=Query.Active;
  btnDel.enabled:=Query.Active and not Query.IsEmpty;
end;

procedure TFEditRefDivers.btnAddClick(Sender:TObject);
begin
  Query.Insert;
  SendFocus(dxDBGrid1);
end;

procedure TFEditRefDivers.QueryAfterInsert(DataSet:TDataSet);
begin
  doRefreshControls;
end;

procedure TFEditRefDivers.btnDelClick(Sender:TObject);
begin
  Query.Delete;
end;

procedure TFEditRefDivers.QueryBeforeDelete(DataSet:TDataSet);
begin
  if MyMessageDlg(rs_Do_you_delete_this_label
    ,mmtConfirmation, mmbYesNo,Self)<>mrYes then
    Abort;
end;

procedure TFEditRefDivers.QueryAfterDelete(DataSet:TDataSet);
begin
  doRefreshControls;
  SendFocus(dxDBGrid1);
end;

procedure TFEditRefDivers.QueryBeforePost(DataSet:TDataSet);
begin
  if Trim(DataSet.FieldByName('REF_LIBELLE').AsString)='' then
  begin
    MyMessageDlg(rs_Error_label_not_to_be_empty_or_spaces_filled
      ,mmtError, [mmbOk],Self);
    SendFocus(dxDBGrid1);
    Abort;
  end
  else
  begin
    if DataSet.FieldByName('REF_LANGUE').AsString='' then
      DataSet.FieldByName('REF_LANGUE').AsString:=gci_context.Langue;
  end;
end;

procedure TFEditRefDivers.DoClose(
  var Action:TCloseAction);
begin
  if Query.Active and(Query.State in [dsEdit,dsInsert]) then
    Query.Post;
  dm.IBTrans_Secondaire.CommitRetaining;
  Query.Close;
  inherited;
end;

procedure TFEditRefDivers.ControlKeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  case Key of
    VK_ESCAPE:
      begin
        dm.IBTrans_Secondaire.RollbackRetaining;
        OpenQuery;
      end;
  end;
end;

procedure TFEditRefDivers.btnExportClick(Sender:TObject);
var
  aFExportTable2TextFile:TFExportTable2TextFile;
begin
  aFExportTable2TextFile:=TFExportTable2TextFile.create(self);
  try
    aFExportTable2TextFile.TableName:=NomTable;
    aFExportTable2TextFile.PropositionFileNameExport:=NomTable+'.txt';
    aFExportTable2TextFile.ShowModal;
  finally
    aFExportTable2TextFile.Free;
  end;
end;

procedure TFEditRefDivers.btnImportClick(Sender:TObject);
var
  aFImportTextFile2Table:TFImportTextFile2Table;
begin
  aFImportTextFile2Table:=TFImportTextFile2Table.create(self);
  try
    aFImportTextFile2Table.TableName:=NomTable;
    aFImportTextFile2Table.ParamsTable.Add('REF_LANGUE');
    aFImportTextFile2Table.ParamsTable.Add('REF_LIBELLE');
    aFImportTextFile2Table.chbVider.Checked:=true;
    if aFImportTextFile2Table.ShowModal=mrOk then
      OpenQuery;
  finally
    aFImportTextFile2Table.free;
  end;
end;

end.

