{-----------------------------------------------------------------------}
{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{ Ancestromania                                                         }
{ Création u_form_edit_ref_temoins André Langlet 2009                 }
{-----------------------------------------------------------------------}

unit u_form_edit_ref_temoins;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  Windows,
{$ELSE}
  LCLType,
{$ENDIF}
  U_FormAdapt,Forms,
  DB,U_ExtDBGrid,
  StdCtrls,Controls,Classes,ExtCtrls,
  u_buttons_appli,
  IBTable;

type

  { TFEditRefTemoins }

  TFEditRefTemoins=class(TF_FormAdapt)
    btnAdd: TFWAdd;
    btnDel: TFWDelete;
    DataSource:TDataSource;
    lbTitle1: TLabel;
    lbTitle2: TLabel;
    Panel2: TPanel;
    Panel3: TPanel;
    pBorder:TPanel;
    Panel7:TPanel;
    Panel4:TPanel;
    lbTitle:TLabel;
    Panel8:TPanel;
    Panel6:TPanel;
    Panel1:TPanel;
    dxDBGrid1:TExtDBGrid;
    btnImport:TFWImport;
    btnExport:TFWExport;
    btnFermer:TFWClose;
    
    Query:TIBTable;
{$IFNDEF FPC}
    QueryREF_RELA_CLEF:TLongintField;
    QueryREF_RELA_LIBELLE:TStringField;
    QueryLANGUE:TStringField;
    QueryREF_RELA_TAG:TStringField;
{$ENDIF}
    procedure btnAddClick(Sender:TObject);
    procedure btnDelClick(Sender:TObject);
    procedure btnExportClick(Sender:TObject);
    procedure btnImportClick(Sender:TObject);
    procedure QueryAfterInsert(DataSet:TDataSet);
    procedure QueryBeforeDelete(DataSet:TDataSet);
    procedure QueryAfterDelete(DataSet:TDataSet);
    procedure QueryBeforePost(DataSet:TDataSet);
  private
    procedure OpenQuery;
  protected
    procedure DoClose(var Action: TCloseAction); override;
    procedure ControlKeyDown(var Key:Word;
      Shift:TShiftState); override;
  public
    procedure Loaded; override;
    procedure DoShowFirstTime; override;
    procedure DoRefreshControls; override;

  end;

implementation

uses  u_dm,
      u_common_functions,
      fonctions_dialogs,
      u_common_ancestro,
      u_common_const,
      u_common_ancestro_functions,
      u_form_export_table_2_textfile,
      u_form_Import_textfile_2_table,
      u_genealogy_context,
      fonctions_ini,
      Dialogs,
      SysUtils;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}

{ TFEditRefTemoins }

procedure TFEditRefTemoins.Loaded;
begin
  f_GetMemIniFile();
  Panel7.Color:=gci_context.ColorLight;
  btnAdd.Color:=gci_context.ColorLight;
  btnDel.Color:=gci_context.ColorLight;
  btnAdd.ColorFrameFocus:=gci_context.ColorLight;
  btnDel.ColorFrameFocus:=gci_context.ColorLight;
  inherited;
end;

procedure TFEditRefTemoins.DoShowFirstTime;
begin
  Caption:=rs_Caption_Roles_and_Relationships_of_the_witnesses;
  OpenQuery;
end;

procedure TFEditRefTemoins.OpenQuery;
begin
  Query.DisableControls;
  try
    Query.Close;
    Query.Filtered:=false;
    Query.Open;
    Query.Filter:='LANGUE='''+gci_context.Langue+'''';
    Query.Filtered:=true;
  finally
    Query.EnableControls;
  end;
  doRefreshControls;
end;

procedure TFEditRefTemoins.DoRefreshControls;
begin
  btnAdd.enabled:=Query.Active;
  btnDel.enabled:=Query.Active and not Query.IsEmpty;
end;

procedure TFEditRefTemoins.btnAddClick(Sender:TObject);
begin
  Query.Insert;
  SendFocus(dxDBGrid1);
end;

procedure TFEditRefTemoins.QueryAfterInsert(DataSet:TDataSet);
begin
  doRefreshControls;
end;

procedure TFEditRefTemoins.btnDelClick(Sender:TObject);
begin
  Query.Delete;
end;

procedure TFEditRefTemoins.QueryBeforeDelete(DataSet:TDataSet);
begin
  if MyMessageDlg(rs_Do_you_delete_this_label
    ,mmtConfirmation, mmbYesNo,Self)<>mrYes then
    Abort;
end;

procedure TFEditRefTemoins.QueryAfterDelete(DataSet:TDataSet);
begin
  doRefreshControls;
  SendFocus(dxDBGrid1);
end;

procedure TFEditRefTemoins.QueryBeforePost(DataSet:TDataSet);
begin
  if Trim(DataSet.FieldByName('REF_RELA_LIBELLE').AsString)='' then
  begin
    MyMessageDlg(rs_Error_label_not_to_be_empty_or_spaces_filled
      ,mmtError, [mmbOk],Self);
    SendFocus(dxDBGrid1);
    Abort;
  end
  else
  begin
    if DataSet.FieldByName('LANGUE').AsString='' then
      DataSet.FieldByName('LANGUE').AsString:=gci_context.Langue;
  end;
end;

procedure TFEditRefTemoins.DoClose(
  var Action:TCloseAction);
begin
  if Query.Active and(Query.State in [dsEdit,dsInsert]) then
    Query.Post;
  dm.IBTrans_Secondaire.CommitRetaining;
  Query.Close;
  inherited;
end;

procedure TFEditRefTemoins.ControlKeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  case Key of
    VK_ESCAPE:
      begin
        dm.IBTrans_Secondaire.RollbackRetaining;
        OpenQuery;
      end;
  end;
end;

procedure TFEditRefTemoins.btnExportClick(Sender:TObject);
var
  aFExportTable2TextFile:TFExportTable2TextFile;
begin
  aFExportTable2TextFile:=TFExportTable2TextFile.create(self);
  try
    aFExportTable2TextFile.TableName:='REF_RELA_TEMOINS';
    aFExportTable2TextFile.PropositionFileNameExport:='REF_RELA_TEMOINS.txt';
    aFExportTable2TextFile.ShowModal;
  finally
    aFExportTable2TextFile.Free;
  end;
end;

procedure TFEditRefTemoins.btnImportClick(Sender:TObject);
var
  aFImportTextFile2Table:TFImportTextFile2Table;
begin
  aFImportTextFile2Table:=TFImportTextFile2Table.create(self);
  try
    aFImportTextFile2Table.TableName:='REF_RELA_TEMOINS';
    aFImportTextFile2Table.NameGeneratorPrimaryIndex:='GEN_REF_RELA_CLEF';
    aFImportTextFile2Table.NamePrimaryIndex:='REF_RELA_CLEF';
    
    aFImportTextFile2Table.ParamsTable.Add('REF_RELA_CLEF');
    aFImportTextFile2Table.ParamsTable.Add('REF_RELA_LIBELLE');
    aFImportTextFile2Table.ParamsTable.Add('LANGUE');
    aFImportTextFile2Table.ParamsTable.Add('REF_RELA_TAG');
    aFImportTextFile2Table.chbVider.Checked:=true;
    if aFImportTextFile2Table.ShowModal=mrOk then
      OpenQuery;
  finally
    aFImportTextFile2Table.free;
  end;
end;

end.

