{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_form_export_geneanet;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

interface

uses
{$IFNDEF FPC}
  jpeg, Windows,
{$ELSE}
{$ENDIF}
  U_FormAdapt,Dialogs,DB,
  IBCustomDataSet,IBQuery,StdCtrls,DBGrids,ExtCtrls,u_buttons_appli, Classes;

type

  { TFExportGeneanet }

  TFExportGeneanet=class(TF_FormAdapt)
    FWSee: TFWFolder;
    IBQPatronymes:TIBQuery;
    IBQPatronymesNOM:TIBStringField;
    DSPatronymes:TDataSource;
    QGeneanet:TIBQuery;
    Panel1:TPanel;
    Panel2:TPanel;
    Panel3:TPanel;
    Panel4:TPanel;
    Panel5:TPanel;
    grid:TDBGrid;
    Panel6:TPanel;
    Label37:TLabel;
    Panel7:TPanel;
    Image1:TImage;
    SaveDialog:TSaveDialog;
    
    btnExport:TFWExport;
    Memo1:TMemo;
    procedure btnExportClick(Sender:TObject);
    procedure FWSeeClick(Sender: TObject);
    procedure GridCellClick(Column: TColumn);
    procedure IBQPatronymesAfterScroll(DataSet: TDataSet);
    procedure Image1Click(Sender:TObject);
  private

  public
    procedure Loaded; override;
    destructor Destroy; override;
    procedure DoRefreshControls; override;
    procedure doPrepare;
  end;

implementation

uses u_dm,
  u_objet_TCut,u_common_functions,
  u_common_ancestro,
  lazutf8classes,
  fonctions_ini,
  fonctions_dialogs,
  fonctions_system,
  u_genealogy_context,
  u_common_const;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}

{ TFExportGeneanet }

procedure TFExportGeneanet.doPrepare;
begin
  IBQPatronymes.DisableControls;
  IBQPatronymes.Close;
  IBQPatronymes.Params[0].AsInteger:=dm.NumDossier;
  IBQPatronymes.Open;
  IBQPatronymes.Last;
  IBQPatronymes.First;
  IBQPatronymes.EnableControls;
  doRefreshControls;
end;

procedure TFExportGeneanet.btnExportClick(Sender:TObject);
var
  k,tot:integer;
  fContent:TStringlistUTF8;
  fCut:TCut;
begin
  tot:=0;
  btnExport.Enabled:=false;
  if (IBQPatronymes.Active)and(not IBQPatronymes.IsEmpty) then
  begin
    fContent:=TStringlistUTF8.create;

    fCut:=TCut.create;
    fCut.ParamSeparator:=';';
    fCut.SetTotalStringParams(10);

    IBQPatronymes.DisableControls;
    with QGeneanet do
    try
      doShowWorking(rs_Please_Wait);
      try
        Close;
        ParamByName( IBQ_I_DOSSIER ).AsInteger:=dm.NumDossier;
        for k:=0 to grid.SelectedRows.Count-1 do
        begin
          IBQPatronymes.GotoBookMark(grid.SelectedRows [k]);
          ParamByName( IBQ_S_PATRONYME ).AsString:=IBQPatronymesNOM.AsString;
          Open;
          with fCut do
          while not Eof do
          begin
            Params[0].AsString:=IBQPatronymesNOM.AsString;
            Params[1].AsString:='';
            Params[2].AsString:=FieldByName('DATE_DEBUT').AsString;
            Params[3].AsString:=FieldByName('DATE_FIN').AsString;
            Params[4].AsString:=FieldByName('COMBIEN').AsString;
            Params[5].AsString:=FieldByName('VILLE').AsString;
            Params[6].AsString:=FieldByName('CP').AsString;
            Params[7].AsString:='';
            Params[8].AsString:=FieldByName('PAYS_CODE').AsString;
            Params[9].AsString:='F';

            ParamsToStr;
            fContent.Add(Line);
            inc(tot);

            Next;
          end;
          Close;
        end;
      finally
        doCloseWorking;
      end;
      if tot>0 then
      begin
        SaveDialog.InitialDir:=gci_context.PathImportExport;

        if SaveDialog.Execute then
        begin
          fContent.SaveToFile(SaveDialog.FileName);
          MyMessageDlg(rs_Info_Geneanet_Export_is_a_success_Import_it_on_geneanet_org
            ,mmtInformation, [mmbOK]);
        end;
      end
      else
      begin
        MyMessageDlg(rs_Info_Nobody_to_export,mmtInformation, [mmbOK]);
      end;
    finally
      IBQPatronymes.EnableControls;
      fCut.Free;
      fContent.free;
    end;
  end;
  DoRefreshControls;
end;

destructor TFExportGeneanet.Destroy;
begin
  grid.SelectedRows.Clear;
  inherited;
end;

procedure TFExportGeneanet.FWSeeClick(Sender: TObject);
begin
  p_OpenFileOrDirectory(gci_context.PathImportExport);
end;

procedure TFExportGeneanet.GridCellClick(Column: TColumn);
begin
  grid.SelectedRows.currentRowSelected := not grid.SelectedRows.currentRowSelected;
end;

procedure TFExportGeneanet.IBQPatronymesAfterScroll(DataSet: TDataSet);
begin
  doRefreshControls;
end;

procedure TFExportGeneanet.Image1Click(Sender:TObject);
begin
  GotoThisURL('www.geneanet.org');
end;

procedure TFExportGeneanet.DoRefreshControls;
begin
  btnExport.enabled:=(IBQPatronymes.Active)and(not IBQPatronymes.IsEmpty)
    and(Grid.SelectedRows.Count>0);
end;

procedure TFExportGeneanet.Loaded;
begin
  f_GetMemIniFile();
  Color:=gci_context.ColorLight;
  Memo1.Color:=gci_context.ColorLight;
  inherited;
end;

end.

