unit u_form_individu_Sa_Vie;
{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}


{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}
{$i ../extends.inc}

interface

uses
{$IFNDEF FPC}
  jpeg, CommCtrl, Windows,
{$ELSE}
{$ENDIF}
  U_FormAdapt,Forms,StdCtrls,
  ExtCtrls,Controls,Classes,SysUtils,Graphics,ComCtrls,
  Menus,PrintersDlgs,
{$IFDEF VIRTUALTREES}
  VirtualTrees,
{$ENDIF}
  IBSQL;

type
  TVieTree=packed record
    {$IFDEF VIRTUALTREES}
    Libelle,
    {$ENDIF}
    TypeTable:string;
    ImageIdx,
    cleID,
    Sexe,
    Sosa,
    Onglet,
    Ligne:integer;
  end;
  PVieTree=^TVieTree;

type

  { TFSaVie }

  TFSaVie=class(TF_FormAdapt)
    Panel3:TPanel;
    dxComponentPrinter1:TPrinterSetupDialog;
    lNom:TLabel;
    Label8:TLabel;
    pmVie:TPopupMenu;
    mOuvreDocument:TMenuItem;
    {$IFDEF VIRTUALTREES}
    tvSaVie:TVirtualStringTree;
    {$ELSE}
    tvSaVie:TTreeView;
    {$ENDIF}
    N1:TMenuItem;
    mOuvre:TMenuItem;
    ImageList1:TImageList;
    reqConjoints:TIBSQL;
    reqEnfants:TIBSQL;
    reqEvFam:TIBSQL;
    reqMedias:TIBSQL;
    reqTemoins:TIBSQL;
    reqEvInd:TIBSQL;
    reqDom:TIBSQL;
    reqMediasInd:TIBSQL;
    mVisualiseMedia:TMenuItem;
    mOuvrirFiche:TMenuItem;
    procedure mOuvreDocumentClick(Sender:TObject);
    procedure pmViePopup(Sender:TObject);
    procedure mOuvreClick(Sender:TObject);
    {$IFDEF VIRTUALTREES}
    procedure tvSaVieBeforeCellPaint(Sender: {$IFDEF VIRTUALTREES}TBaseVirtualTree{$ELSE}TTreeview{$ENDIF};
      TargetCanvas: TCanvas; Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}; Column: TColumnIndex;
      CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
    procedure tvSaVieGetText(Sender: {$IFDEF VIRTUALTREES}TBaseVirtualTree{$ELSE}TTreeview{$ENDIF}; Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF};
      Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
    {$ENDIF}
    procedure tvSaVieDblClick(Sender:TObject);
    procedure tvSaVieCustomDrawItem(Sender:TCustomTreeView;
      Node:TTreeNode;State:TCustomDrawState;var DefaultDraw:Boolean);
    procedure mVisualiseMediaClick(Sender:TObject);
    procedure mOuvrirFicheClick(Sender:TObject);
    procedure tvSaVieExpanding(Sender: {$IFDEF VIRTUALTREES}TBaseVirtualTree{$ELSE}TObject{$ENDIF}; Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF};
      var Allowed: Boolean);
    procedure tvSaVieGetImageIndex(Sender: {$IFDEF VIRTUALTREES}TBaseVirtualTree{$ELSE}TObject{$ENDIF};
      Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}{$IFDEF VIRTUALTREES}; Kind: TVTImageKind; Column: TColumnIndex;
      var Ghosted: Boolean; var ImageIndex: Integer{$ENDIF});
    procedure tvSaVieMouseMove(Sender:TObject;Shift:TShiftState;X,
      Y:Integer);
    procedure tvSaVieMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);

  private
    { Déclarations privées }
    iCleIndi:integer;
    fCanDeletionTV,bBloque:boolean;
    Xmouse:integer;
    Acte:TVieTree;
    procedure OuvreObjetPointe;
  protected
    procedure DoClose(var Action: TCloseAction); override;
  public
    { Déclarations publiques }
    bModified:boolean;
    procedure Loaded; override;
    procedure DoShow; override;
    procedure DoShowFirstTime; override;
    function doOpenQuerys:boolean;
    procedure doInit(iClef:integer;sNom:string);
  end;


implementation

uses u_common_functions,
     u_common_ancestro,
     u_common_resources,
     u_Common_const,
     u_Dm,u_form_individu,
     fonctions_string,
     fonctions_ini,
     fonctions_vtree,
     fonctions_reports,
     u_common_ancestro_functions,
     u_Form_Main,u_genealogy_context,Types;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}
var    fFormIndividu:TFIndividu;

{$IFNDEF VIRTUALTREES}
procedure ValidateNode ( ANode : TTreeNode );
var Vie : PVieTree;
Begin
  New(Vie);
  with ANode do
   Begin
    Data:=Vie;
   end;
end;
{$ENDIF}
procedure TFSaVie.doInit(iClef:integer;sNom:string);
var
  iNoeudparent,iNoeud,iNoeudConjoint,iNoeudEve,iAjoute,iNoeudTitre:{$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF};
  lsNom,naissance,deces:string;
  NumOrdre:integer;
begin
  tvSaVie.DisableAlign;
  {$IFDEF VIRTUALTREES}
  tvSaVie.NodeDataSize:=SizeOf(TVieTree)+1;
  {$ENDIF}
  iCleIndi:=iClef;
  bModified:=false;
  tvSaVie.{$IFNDEF VIRTUALTREES}Items.{$ENDIF}Clear;
  reqConjoints.Close;
  reqEnfants.Close;
  reqEvFam.Close;
  reqMedias.Close;
  reqTemoins.Close;
  reqEvInd.Close;
  reqMediasInd.Close;

  lsNom:=FFormIndividu.GetNomIndividu;
  if length(FFormIndividu.sDateNaissance)>0 then
  begin
    naissance:=FFormIndividu.sDateNaissance;
    if length(FFormIndividu.sVilleNaissance)>0 then
      naissance:=naissance+'-'+FFormIndividu.sVilleNaissance;
  end
  else if length(FFormIndividu.sVilleNaissance)>0 then
    naissance:=FFormIndividu.sVilleNaissance
  else
    naissance:='';
  if length(FFormIndividu.sDateDeces)>0 then
  begin
    deces:=FFormIndividu.sDateDeces;
    if length(FFormIndividu.sVilleDeces)>0 then
      deces:=deces+'-'+FFormIndividu.sVilleDeces;
  end
  else if length(FFormIndividu.sVilleDeces)>0 then
    deces:=FFormIndividu.sVilleDeces
  else
    deces:='';
  lsNom:=lsNom+GetStringNaissanceDeces(naissance,deces);

  iNoeudparent:=tvSaVie.{$IFNDEF VIRTUALTREES}Items.{$ENDIF}AddChild(nil,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
  {$IFDEF VIRTUALTREES}
  with tvSaVie do
  {$ENDIF}
    ValidateNode(iNoeudparent{$IFDEF VIRTUALTREES},False{$ENDIF});
  {$IFNDEF VIRTUALTREES}
  {$ENDIF}

  with PVieTree({$IFDEF VIRTUALTREES}tvSaVie.GetNodeData(iNoeudparent){$ELSE}iNoeudparent.Data{$ENDIF})^,FFormIndividu.QueryIndividu do
   Begin
    cleID:=1;
    Sexe:=FieldByName('SEXE').AsInteger;
    TypeTable:='INDIVIDU';
    Onglet:=_ONGLET_IDENTITE;
    if FieldByName('NUM_SOSA').AsFloat>0 then
      Sosa:=1
    else
      Sosa:=0;

    {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeudparent.Text{$ENDIF}:=lsNom;
    ImageIdx:=19;
//    iNoeudparent.SelectedIndex:=19;
//    iNoeudparent.StateIndex:=-1;
   end;

  with FFormIndividu.aFIndividuIdentite.IBQmariageParents do
  begin
    Params[0].AsInteger:=FFormIndividu.QueryIndividuCLE_FICHE.AsInteger;
    ExecQuery;
    if not Eof then
    with tvSaVie do
     begin
      iNoeud:={$IFNDEF VIRTUALTREES}Items.{$ENDIF}AddChild(iNoeudparent,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
      ValidateNode(iNoeud{$IFDEF VIRTUALTREES},False{$ENDIF});
      {$IFNDEF VIRTUALTREES}
      {$ENDIF}
      if length(FieldByName('NOM_PERE').AsString)>0 then
      with PVieTree({$IFDEF VIRTUALTREES}GetNodeData(iNoeud){$ELSE}iNoeud.Data{$ENDIF})^ do
      begin
        cleID:=FFormIndividu.QueryIndividuCLE_PERE.AsInteger;
        Sexe:=1;
        TypeTable:='INDIVIDU';
        Onglet:=_ONGLET_IDENTITE;
        if FieldByName('PERE_SOSA').AsFloat>0 then
          Sosa:=1
        else
          Sosa:=0;
        lsNom:=fs_RemplaceMsg(rs_Father,[FieldByName('NOM_PERE').AsString]);
        if length(FieldByName('PRENOM_PERE').AsString)>0 then
          lsNom:=lsNom+', '+FieldByName('PRENOM_PERE').AsString;
        if length(FieldByName('PERE_NAISSANCE').AsString)>0 then
        begin
          naissance:=FieldByName('PERE_NAISSANCE').AsString;
          if length(FieldByName('PERE_VILLE_NAISSANCE').AsString)>0 then
            naissance:=naissance+'-'+FieldByName('PERE_VILLE_NAISSANCE').AsString;
        end
        else if length(FieldByName('PERE_VILLE_NAISSANCE').AsString)>0 then
          naissance:=FieldByName('PERE_VILLE_NAISSANCE').AsString
        else
          naissance:='';
        if length(FieldByName('PERE_DECES').AsString)>0 then
        begin
          deces:=FieldByName('PERE_DECES').AsString;
          if length(FieldByName('PERE_VILLE_DECES').AsString)>0 then
            deces:=deces+'-'+FieldByName('PERE_VILLE_DECES').AsString;
        end
        else if length(FieldByName('PERE_VILLE_DECES').AsString)>0 then
          deces:=FieldByName('PERE_VILLE_DECES').AsString
        else
          deces:='';
        lsNom:=lsNom+GetStringNaissanceDeces(naissance,deces);
        {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeud.Text{$ENDIF} := lsNom;
        ImageIdx:=29;
//        iNoeud.SelectedIndex:=29;
//        iNoeud.StateIndex:=-1;
      end;

      iNoeud:={$IFNDEF VIRTUALTREES}Items.{$ENDIF}AddChild(iNoeudparent,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
      ValidateNode(iNoeud{$IFDEF VIRTUALTREES},False{$ENDIF});
      {$IFNDEF VIRTUALTREES}
      {$ENDIF}
      if length(FieldByName('NOM_MERE').AsString)>0 then
      with PVieTree({$IFDEF VIRTUALTREES}GetNodeData(iNoeud){$ELSE}iNoeud.Data{$ENDIF})^ do
      begin
        cleID:=FFormIndividu.QueryIndividuCLE_MERE.AsInteger;
        Sexe:=2;
        TypeTable:='INDIVIDU';
        Onglet:=_ONGLET_IDENTITE;
        if FieldByName('MERE_SOSA').AsFloat>0 then
          Sosa:=1
        else
          Sosa:=0;
        lsNom:=fs_RemplaceMsg(rs_Mother,[FieldByName('NOM_MERE').AsString]);
        if length(FieldByName('PRENOM_MERE').AsString)>0 then
          lsNom:=lsNom+', '+FieldByName('PRENOM_MERE').AsString;
        if length(FieldByName('MERE_NAISSANCE').AsString)>0 then
        begin
          naissance:=FieldByName('MERE_NAISSANCE').AsString;
          if length(FieldByName('MERE_VILLE_NAISSANCE').AsString)>0 then
            naissance:=naissance+'-'+FieldByName('MERE_VILLE_NAISSANCE').AsString;
        end
        else if length(FieldByName('MERE_VILLE_NAISSANCE').AsString)>0 then
          naissance:=FieldByName('MERE_VILLE_NAISSANCE').AsString
        else
          naissance:='';
        if length(FieldByName('MERE_DECES').AsString)>0 then
        begin
          deces:=FieldByName('MERE_DECES').AsString;
          if length(FieldByName('MERE_VILLE_DECES').AsString)>0 then
            deces:=deces+'-'+FieldByName('MERE_VILLE_DECES').AsString;
        end
        else if length(FieldByName('MERE_VILLE_DECES').AsString)>0 then
          deces:=FieldByName('MERE_VILLE_DECES').AsString
        else
          deces:='';
        lsNom:=lsNom+GetStringNaissanceDeces(naissance,deces);
        {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeud.Text{$ENDIF} := lsNom;
        ImageIdx:=30;
//        iNoeud.SelectedIndex:=30;
  //      iNoeud.StateIndex:=-1;
      end;
    end;
    Close;
  end;

  reqConjoints.ParamByName( IBQ_INDI ).AsInteger:=FFormIndividu.QueryIndividuCLE_FICHE.AsInteger;
  reqConjoints.ExecQuery;
  NumOrdre:=-1;
  if not reqConjoints.Eof then
  with tvSaVie{$IFNDEF VIRTUALTREES},Items{$ENDIF} do
  begin
    lsNom:=rs_Joint;
    iNoeudTitre:=AddChild(iNoeudparent,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
    ValidateNode(iNoeudTitre{$IFDEF VIRTUALTREES},False{$ENDIF});
    {$IFNDEF VIRTUALTREES}
    {$ENDIF}
    with PVieTree({$IFDEF VIRTUALTREES}GetNodeData(iNoeudTitre){$ELSE}iNoeudTitre.Data{$ENDIF})^ do
      Begin
        {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeudTitre.Text{$ENDIF} := lsNom;
        ImageIdx:=23;
        cleID:=1;
        Sexe:=0;
        TypeTable:='T_UNION';
        Onglet:=_ONGLET_SES_CONJOINTS;
        Sosa:=0;
        Ligne:=-1;
      End;
//    iNoeudTitre.SelectedIndex:=iNoeudTitre.ImageIndex;
  //  iNoeudTitre.StateIndex:=-1;
    repeat
      iNoeudConjoint:=AddChild(iNoeudTitre,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
      ValidateNode(iNoeudConjoint{$IFDEF VIRTUALTREES},False{$ENDIF});
      {$IFNDEF VIRTUALTREES}
      {$ENDIF}
      with PVieTree({$IFDEF VIRTUALTREES}GetNodeData(iNoeudConjoint){$ELSE}iNoeudConjoint.Data{$ENDIF})^ do
        Begin
          cleID:=reqConjoints.FieldByName('CLE_FICHE').AsInteger;
          Sexe:=reqConjoints.FieldByName('SEXE').AsInteger;
          TypeTable:='T_UNION';
          Sosa:=reqConjoints.FieldByName('SOSA').AsInteger;
          if reqConjoints.FieldByName('CLE_FICHE').AsInteger>0 then
          begin
            NumOrdre:=reqConjoints.FieldByName('CLE_FICHE').AsInteger;
            Ligne:=NumOrdre;
            Onglet:=_ONGLET_SES_CONJOINTS;
          end
          else
          begin
            Ligne:=-1;
            Onglet:=_ONGLET_IDENTITE;
          end;
          lsNom:=reqConjoints.FieldByName('NOM').AsString;
          if length(reqConjoints.FieldByName('PRENOM').AsString)>0 then
            lsNom:=lsNom+', '+reqConjoints.FieldByName('PRENOM').AsString;
          naissance:=reqConjoints.FieldByName('NAISSANCE').AsString;
          deces:=reqConjoints.FieldByName('DECES').AsString;
          {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeudConjoint.Text{$ENDIF}:=lsNom+GetStringNaissanceDeces(naissance,deces);
          case reqConjoints.FieldByName('SEXE').AsInteger of
            1: ImageIdx:=29;
            2: ImageIdx:=30;
          end;
         End;

//      iNoeudConjoint.SelectedIndex:=iNoeudConjoint.ImageIndex;
//      iNoeudConjoint.StateIndex:=-1;

      reqEnfants.ParamByName( IBQ_INDI ).AsInteger:=FFormIndividu.QueryIndividuCLE_FICHE.AsInteger;
      reqEnfants.ParamByName( IBQ_CONJOINT ).Value:=reqConjoints.FieldByName('CLE_FICHE').Value;
      reqEnfants.ExecQuery;
      with reqEnfants do
      while not Eof do
      begin
       iNoeud:=AddChild(iNoeudConjoint,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
       ValidateNode(iNoeud{$IFDEF VIRTUALTREES},False{$ENDIF});
       {$IFNDEF VIRTUALTREES}
       {$ENDIF}
        with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeud ){$ELSE}iNoeud.Data{$ENDIF})^ do
          Begin
            cleID:=FieldByName('CLE_FICHE').AsInteger;
            Sexe:=FieldByName('SEXE').AsInteger;
            TypeTable:='INDIVIDU';
            Onglet:=_ONGLET_IDENTITE;
            Sosa:=FieldByName('SOSA').AsInteger;
            lsNom:=FieldByName('NOM').AsString;
            if length(FieldByName('PRENOM').AsString)>0 then
              lsNom:=lsNom+', '+FieldByName('PRENOM').AsString;
            naissance:=FieldByName('NAISSANCE').AsString;
            deces:=FieldByName('DECES').AsString;
            {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeud.Text{$ENDIF}:=lsNom+GetStringNaissanceDeces(naissance,deces);
            case FieldByName('SEXE').AsInteger of
              1: ImageIdx:=24;
              2: ImageIdx:=25;
              else
                ImageIdx:=31;
            end;
          End;
//        iNoeud.SelectedIndex:=iNoeud.ImageIndex;
//        iNoeud.StateIndex:=-1;
        Next;
      end;
      reqEnfants.Close;

      reqEvFam.ParamByName( IBQ_INDI ).AsInteger:=FFormIndividu.QueryIndividuCLE_FICHE.AsInteger;
      reqEvFam.ParamByName( IBQ_CONJOINT ).Value:=reqConjoints.FieldByName('CLE_FICHE').Value;
      reqEvFam.ExecQuery;
      while not reqEvFam.Eof do
       Begin
        iNoeudEve:=AddChild(iNoeudConjoint,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
        ValidateNode(iNoeudEve{$IFDEF VIRTUALTREES},False{$ENDIF});
        {$IFNDEF VIRTUALTREES}
        {$ENDIF}
        with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeudEve ){$ELSE}iNoeudEve.Data{$ENDIF})^ do
        begin
          cleID:=reqEvFam.FieldByName('CLEF').AsInteger;
          Sexe:=0;
          TypeTable:='T_UNION';
          Onglet:=_ONGLET_SES_CONJOINTS;
          Sosa:=0;
          Ligne:=NumOrdre;
          {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeudEve.Text{$ENDIF}:=reqEvFam.FieldByName('libelle').AsString;
          ImageIdx:=reqEvFam.FieldByName('categorie').AsInteger;

//        iNoeudEve.SelectedIndex:=iNoeudEve.ImageIndex;
    //    iNoeudEve.StateIndex:=reqEvFamstatus.AsInteger+7;
        end;

        reqMedias.ParamByName( IBQ_table ).AsString:='F';
        reqMedias.ParamByName( IBQ_eve ).AsInteger:=reqEvFam.FieldByName('CLEF').AsInteger;
        reqMedias.ExecQuery;
        with reqMedias do
        while not Eof do
        begin
          iNoeud:=AddChild(iNoeudEve,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
          ValidateNode(iNoeud{$IFDEF VIRTUALTREES},False{$ENDIF});
          {$IFNDEF VIRTUALTREES}
          {$ENDIF}
          with PVieTree({$IFDEF VIRTUALTREES}GetNodeData(iNoeud){$ELSE}iNoeud.Data{$ENDIF})^ do
           Begin
            cleID:=FieldByName('CLEF').AsInteger;
            Sexe:=0;
            TypeTable:='T_UNION';
            Onglet:=_ONGLET_SES_CONJOINTS;
            Sosa:=0;
            Ligne:=NumOrdre;
            {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeud.Text{$ENDIF}:=StringReplace(FieldByName('libelle').AsString,_CRLF,' ',[rfReplaceAll]);
            ImageIdx:=FieldByName('type').AsInteger;
           End;
    //          iNoeud.SelectedIndex:=iNoeud.ImageIndex;
        //    iNoeud.StateIndex:=-1;
          Next;
        end;
        reqMedias.Close;

        reqTemoins.ParamByName( IBQ_table ).AsString:='U';
        reqTemoins.ParamByName( IBQ_eve ).AsInteger:=reqEvFam.FieldByName('CLEF').AsInteger;
        reqTemoins.ExecQuery;
        with reqTemoins do
        if not Eof then
        begin
          iAjoute:=AddChild(iNoeudEve,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
          ValidateNode(iAjoute{$IFDEF VIRTUALTREES},False{$ENDIF});
          {$IFNDEF VIRTUALTREES}
          {$ENDIF}
          with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iAjoute ){$ELSE}iAjoute.Data{$ENDIF})^ do
           Begin
            lsNom:=rs_Witness;
            cleID:=1;
            Sexe:=0;
            TypeTable:='T_UNION';
            Onglet:=_ONGLET_SES_CONJOINTS;
            Sosa:=0;
            Ligne:=NumOrdre;
            ImageIdx:=5;
            {$IFDEF VIRTUALTREES}libelle{$ELSE}iAjoute.Text{$ENDIF}:=lsNom;
           end;
//          iAjoute.SelectedIndex:=iAjoute.ImageIndex;
      //    iAjoute.StateIndex:=-1;
          repeat
            iNoeud:=AddChild(iAjoute,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
            ValidateNode(iNoeud{$IFDEF VIRTUALTREES},False{$ENDIF});
            {$IFNDEF VIRTUALTREES}
            {$ENDIF}
            with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeud ){$ELSE}iNoeud.Data{$ENDIF})^ do
             Begin
              cleID:=FieldByName('CLE_FICHE').AsInteger;
              Sexe:=FieldByName('SEXE').AsInteger;
              TypeTable:='T_UNION';
              Onglet:=_ONGLET_SES_CONJOINTS;
              Sosa:=FieldByName('SOSA').AsInteger;
              Ligne:=NumOrdre;
              naissance:=FieldByName('NAISSANCE').AsString;
              deces:=FieldByName('DECES').AsString;
              {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeud.Text{$ENDIF}:=FieldByName('NomPrenom').AsString
               +GetStringNaissanceDeces(naissance,deces)
               +', '+FieldByName('libelle').AsString;
              case FieldByName('SEXE').AsInteger of
                1:ImageIdx:=29;
                2:ImageIdx:=30;
                else
                  ImageIdx:=31;

              end;
             end;
//            iNoeud.SelectedIndex:=iNoeud.ImageIndex;
  //          iNoeud.StateIndex:=-1;
            Next;
          until Eof;
          if RecordCount>1 then
            {$IFDEF VIRTUALTREES}
            PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iAjoute ){$ELSE}iAjoute.Data{$ENDIF})^.Libelle:=rs_Witness;
            {$ELSE}
            iAjoute.Text:=rs_Witness;
            {$ENDIF}
        end;
        reqTemoins.Close;
        reqEvFam.Next;
      end;
      reqEvFam.Close;
      reqConjoints.Next;
    until reqConjoints.Eof;
    if reqConjoints.RecordCount>1 then
      {$IFDEF VIRTUALTREES}
      PVieTree(GetNodeData( iNoeudTitre ))^.Libelle:=rs_Joints;
      {$ELSE}
      iNoeudTitre.Text:=rs_Joints;
      {$ENDIF}
  end;
  reqConjoints.Close;

  reqEvInd.ParamByName( IBQ_indi ).AsInteger:=FFormIndividu.QueryIndividuCLE_FICHE.AsInteger;
  reqEvInd.ExecQuery;
  with tvSaVie{$IFNDEF VIRTUALTREES},Items{$ENDIF} do
  if not reqEvInd.Eof then
   Begin
    iNoeudTitre:=AddChild(iNoeudparent,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
    ValidateNode(iNoeudTitre{$IFDEF VIRTUALTREES},False{$ENDIF});
    {$IFNDEF VIRTUALTREES}
    {$ENDIF}
    with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeudTitre ){$ELSE}iNoeudTitre.Data{$ENDIF})^ do
     begin
      lsNom:=rs_Individual_event;
      cleID:=reqEvInd.FieldByName('clef').AsInteger;
      Sexe:=0;
      TypeTable:='EVENEMENTS_IND';
      Onglet:=_ONGLET_IDENTITE;
      Sosa:=0;
      Ligne:=reqEvInd.FieldByName('clef').AsInteger;
      {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeudTitre.Text{$ENDIF}:=lsNom;
      ImageIdx:=7;
     end;
//    iNoeudTitre.SelectedIndex:=iNoeudTitre.ImageIndex;
//    iNoeudTitre.StateIndex:=-1;
    repeat
      iNoeudEve:=AddChild(iNoeudTitre,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
      ValidateNode(iNoeudEve{$IFDEF VIRTUALTREES},False{$ENDIF});
      {$IFNDEF VIRTUALTREES}
      {$ENDIF}
      with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeudEve ){$ELSE}iNoeudEve.Data{$ENDIF})^ do
       begin
        cleID:=reqEvInd.FieldByName('clef').AsInteger;
        Sexe:=0;
        TypeTable:='EVENEMENTS_IND';
        Onglet:=_ONGLET_IDENTITE;
        Sosa:=0;
        Ligne:=reqEvInd.FieldByName('clef').AsInteger;
        {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeudEve.Text{$ENDIF}:=reqEvInd.FieldByName('libelle').AsString;
        ImageIdx:=reqEvInd.FieldByName('categorie').AsInteger;

//      iNoeudEve.SelectedIndex:=iNoeudEve.ImageIndex;
  //    iNoeudEve.StateIndex:=reqEvIndstatus.AsInteger+7;
       end;

      reqMedias.ParamByName( IBQ_table ).AsString:='I';
      reqMedias.ParamByName( IBQ_eve ).AsInteger:=reqEvInd.FieldByName('CLEF').AsInteger;
      reqMedias.ExecQuery;
      with reqMedias do
      while not Eof do
       begin
        iNoeud:=AddChild(iNoeudEve,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
        ValidateNode(iNoeud{$IFDEF VIRTUALTREES},False{$ENDIF});
        {$IFNDEF VIRTUALTREES}
        {$ENDIF}
        with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeud ){$ELSE}iNoeud.Data{$ENDIF})^ do
         begin
          cleID:=FieldByName('clef').AsInteger;
          Sexe:=0;
          TypeTable:='EVENEMENTS_IND';
          Onglet:=_ONGLET_IDENTITE;
          Sosa:=0;
          Ligne:=reqEvInd.FieldByName('clef').AsInteger;
          {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeud.Text{$ENDIF}:=StringReplace(FieldByName('libelle').AsString,_CRLF,' ',[rfReplaceAll]);
          ImageIdx:=FieldByName('type').AsInteger;
         end;
//        iNoeud.SelectedIndex:=iNoeud.ImageIndex;
   //     iNoeud.StateIndex:=-1;
        reqMedias.Next;
       end;
      reqMedias.Close;

      reqTemoins.ParamByName( IBQ_table ).AsString:='I';
      reqTemoins.ParamByName( IBQ_eve ).AsInteger:=reqEvInd.FieldByName('CLEF').AsInteger;
      reqTemoins.ExecQuery;
      with reqTemoins do
      if not Eof then
      begin
        iAjoute:=AddChild(iNoeudEve,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
        ValidateNode(iAjoute{$IFDEF VIRTUALTREES},False{$ENDIF});
        {$IFNDEF VIRTUALTREES}
        {$ENDIF}
        with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iAjoute ){$ELSE}iAjoute.Data{$ENDIF})^ do
         begin
          lsNom:=rs_Witness;
          cleID:=reqEvInd.FieldByName('clef').AsInteger;
          Sexe:=0;
          TypeTable:='EVENEMENTS_IND';
          Onglet:=_ONGLET_IDENTITE;
          Sosa:=0;
          Ligne:=reqEvInd.FieldByName('clef').AsInteger;
          {$IFDEF VIRTUALTREES}libelle{$ELSE}iAjoute.Text{$ENDIF}:=lsNom;
          ImageIdx:=5;
         end;
//        iAjoute.SelectedIndex:=iAjoute.ImageIndex;
 //       iAjoute.StateIndex:=-1;
        repeat
          iNoeud:=AddChild(iAjoute,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
          ValidateNode(iNoeud{$IFDEF VIRTUALTREES},False{$ENDIF});
          {$IFNDEF VIRTUALTREES}
          {$ENDIF}
          with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeud ){$ELSE}iNoeud.Data{$ENDIF})^ do
           begin
            cleID:=FieldByName('cle_fiche').AsInteger;
            Sexe:=FieldByName('SEXE').AsInteger;
            TypeTable:='EVENEMENTS_IND';
            Onglet:=_ONGLET_IDENTITE;
            Sosa:=FieldByName('SOSA').AsInteger;
            Ligne:=reqEvInd.FieldByName('clef').AsInteger;
            naissance:=FieldByName('NAISSANCE').AsString;
            deces:=FieldByName('DECES').AsString;
            {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeud.Text{$ENDIF}:=FieldByName('NomPrenom').AsString
             +GetStringNaissanceDeces(naissance,deces)
              +', '+FieldByName('libelle').AsString;
            case FieldByName('SEXE').AsInteger of
              1:ImageIdx:=29;
              2:ImageIdx:=30;
              else
                ImageIdx:=31;
            end;
           end;
//         iNoeud.SelectedIndex:=iNoeud.ImageIndex;
//          iNoeud.StateIndex:=-1;
          Next;
        until Eof;
        if RecordCount>1 then
          {$IFDEF VIRTUALTREES}
          PVieTree(GetNodeData( iAjoute ))^.Libelle:=rs_Witnesses;
          {$ELSE}
          iAjoute.Text:=rs_Witnesses;
          {$ENDIF}
      end;
      reqTemoins.Close;
      reqEvInd.Next;
    until reqEvInd.Eof;
    if reqEvInd.RecordCount>1 then
      {$IFDEF VIRTUALTREES}
      PVieTree(GetNodeData( iNoeudTitre ))^.Libelle:=rs_Individual_events;
      {$ELSE}
      iNoeudTitre.Text:=rs_Individual_events;
      {$ENDIF}
  end;
  reqEvInd.Close;

  reqDom.ParamByName( IBQ_indi ).AsInteger:=FFormIndividu.QueryIndividuCLE_FICHE.AsInteger;
  reqDom.ExecQuery;
  with tvSaVie{$IFNDEF VIRTUALTREES},Items{$ENDIF} do
  if not reqDom.Eof then
   begin
    iNoeudTitre:=AddChild(iNoeudparent,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
    ValidateNode(iNoeudTitre{$IFDEF VIRTUALTREES},False{$ENDIF});
    {$IFNDEF VIRTUALTREES}
    {$ENDIF}
    with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeudTitre ){$ELSE}iNoeudTitre.Data{$ENDIF})^ do
     begin
      lsNom:='Domicile';
      cleID:=reqDom.FieldByName('clef').AsInteger;
      Sexe:=0;
      TypeTable:='ADRESSES_IND';
      Onglet:=_ONGLET_DOMICILES;
      Sosa:=0;
      Ligne:=0;
      {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeudTitre.Text{$ENDIF}:=lsNom;
      ImageIdx:=28;
     end;
//    iNoeudTitre.SelectedIndex:=iNoeudTitre.ImageIndex;
 //   iNoeudTitre.StateIndex:=-1;
    repeat
      iNoeudEve:=AddChild(iNoeudTitre,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
      ValidateNode(iNoeudEve{$IFDEF VIRTUALTREES},False{$ENDIF});
      {$IFNDEF VIRTUALTREES}
      {$ENDIF}
      with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeudEve ){$ELSE}iNoeudEve.Data{$ENDIF})^ do
       begin
        cleID:=reqDom.FieldByName('clef').AsInteger;
        Sexe:=0;
        TypeTable:='ADRESSES_IND';
        Onglet:=_ONGLET_DOMICILES;
        Sosa:=0;
        Ligne:=reqDom.FieldByName('clef').AsInteger;
        {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeudEve.Text{$ENDIF}:=reqDom.FieldByName('libelle').AsString;
        ImageIdx:=26;
//      iNoeudEve.SelectedIndex:=iNoeudEve.ImageIndex;
     // iNoeudEve.StateIndex:=-1;
       end;

      reqMedias.ParamByName( IBQ_table ).AsString:='I';
      reqMedias.ParamByName( IBQ_eve ).AsInteger:=reqDom.FieldByName('CLEF').AsInteger;
      reqMedias.ExecQuery;
      with reqMedias do
      while not Eof do
      begin
        iNoeud:=AddChild(iNoeudEve,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
        ValidateNode(iNoeud{$IFDEF VIRTUALTREES},False{$ENDIF});
        {$IFNDEF VIRTUALTREES}
        {$ENDIF}
        with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeud ){$ELSE}iNoeud.Data{$ENDIF})^ do
         begin
          cleID:=FieldByName('clef').AsInteger;
          Sexe:=0;
          TypeTable:='ADRESSES_IND';
          Onglet:=_ONGLET_DOMICILES;
          Sosa:=0;
          Ligne:=reqDom.FieldByName('clef').AsInteger;
          {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeud.Text{$ENDIF}:=StringReplace(FieldByName('libelle').AsString,_CRLF,' ',[rfReplaceAll]);
          ImageIdx:=FieldByName('type').AsInteger;
//        iNoeud.SelectedIndex:=iNoeud.ImageIndex;
   //     iNoeud.StateIndex:=-1;
         End;
        Next;
      end;
      reqMedias.Close;

      reqDom.Next;
    until reqDom.Eof;
    if reqDom.RecordCount>1 then
      {$IFDEF VIRTUALTREES}
      PVieTree(GetNodeData( iNoeudTitre ))^.Libelle:=rs_Homes;
      {$ELSE}
      iNoeudTitre.Text:=rs_Homes;
      {$ENDIF}
  end;
  reqDom.Close;

  reqMediasInd.ParamByName( IBQ_indi ).AsInteger:=FFormIndividu.QueryIndividuCLE_FICHE.AsInteger;
  reqMediasInd.ExecQuery;
  with reqMediasInd, tvSaVie{$IFNDEF VIRTUALTREES},Items{$ENDIF} do
  if not Eof then
   begin
    iNoeudTitre:=AddChild(iNoeudparent,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
    ValidateNode(iNoeudTitre{$IFDEF VIRTUALTREES},False{$ENDIF});
    {$IFNDEF VIRTUALTREES}
    {$ENDIF}
    with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeudTitre ){$ELSE}iNoeudTitre.Data{$ENDIF})^ do
     begin
      lsNom:=rs_Individual_Media;
      cleID:=FieldByName('clef').AsInteger;
      Sexe:=0;
      TypeTable:='MEDIA_POINTEURS';
      Onglet:=_ONGLET_PHOTOS_DOCS;
      Sosa:=0;
      Ligne:=FieldByName('mp_clef').AsInteger;
      {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeudTitre.Text{$ENDIF}:=lsNom;
      ImageIdx:=27;
     end;
//    iNoeudTitre.SelectedIndex:=iNoeudTitre.ImageIndex;
//    iNoeudTitre.StateIndex:=-1;
    repeat
      iNoeud:=AddChild(iNoeudTitre,{$IFDEF VIRTUALTREES}nil{$ELSE}''{$ENDIF});
      ValidateNode(iNoeud{$IFDEF VIRTUALTREES},False{$ENDIF});
      with PVieTree({$IFDEF VIRTUALTREES}GetNodeData( iNoeud ){$ELSE}iNoeud.Data{$ENDIF})^ do
       begin
        cleID:=FieldByName('clef').AsInteger;
        Sexe:=0;
        TypeTable:='MEDIA_POINTEURS';
        Onglet:=_ONGLET_PHOTOS_DOCS;
        Sosa:=0;
        Ligne:=FieldByName('mp_clef').AsInteger;
        {$IFDEF VIRTUALTREES}libelle{$ELSE}iNoeud.Text{$ENDIF}:=StringReplace(FieldByName('libelle').AsString,_CRLF,' ',[rfReplaceAll]);
        ImageIdx:=FieldByName('type').AsInteger;
       end;
//      iNoeud.SelectedIndex:=iNoeud.ImageIndex;
  //    iNoeud.StateIndex:=-1;
      Next;
    until Eof;
    if RecordCount>1 then
      {$IFDEF VIRTUALTREES}
      PVieTree(GetNodeData( iNoeudTitre ))^.Libelle:=rs_Individual_Medias;
      {$ELSE}
      iNoeudTitre.Text:=rs_Individual_Medias;
      {$ENDIF}
  end;
  reqMediasInd.Close;

  bBloque:=false;
  tvSaVie.FullExpand;
  bBloque:=true;
  {$IFDEF VIRTUALTREES}
  tvSaVie.FocusedNode := tvSaVie.RootNode.FirstChild;
  {$ELSE}
  tvSaVie.Selected := tvSaVie.Items.GetFirstNode;
  {$ENDIF}
  //tvSaVie.VisiblePath[tvSaVie.RootNode]:=true;
  tvSaVie.EnableAlign;
end;

procedure TFSaVie.DoClose(var Action:TCloseAction);
begin
  inherited;
  Action:=caFree;
end;

procedure TFSaVie.Loaded;
begin
  tvSaVie:= {$IFDEF VIRTUALTREES}TVirtualStringTree{$ELSE}TTreeview{$ENDIF}.Create(Self);
  with tvSaVie do
   Begin
    Parent:=Self;
    {$IFNDEF VIRTUALTREES}
    ReadOnly:=True;
    {$ENDIF}
    Hint := rs_Hint_His_Her_Life;
    Align := alClient;
    {$IFDEF VIRTUALTREES}
    Header.AutoSizeIndex := 0;
    Header.DefaultHeight := 17;
    Header.Height := 17;
    Header.MainColumn := -1;
    TreeOptions.MiscOptions := [toCheckSupport, toFullRepaintOnResize, toGridExtensions, toInitOnSave, toToggleOnDblClick, toWheelPanning, toEditOnClick];
    OnBeforeCellPaint := tvSaVieBeforeCellPaint;
    OnGetText := tvSaVieGetText;
    {$ENDIF}
    Images := dm.ImgCatFondBlanc;
    ParentColor := True;
    ParentShowHint := False;
    PopupMenu := pmVie;
    ShowHint := True;
    StateImages := ImageList1;
    TabOrder := 1;
    OnCollapsing := tvSaVieExpanding;
    OnDblClick := tvSaVieDblClick;
    OnExpanding := tvSaVieExpanding;
    OnGetImageIndex := tvSaVieGetImageIndex;
    OnMouseDown := tvSaVieMouseDown;
    OnMouseMove := tvSaVieMouseMove;
  end;

  f_GetMemIniFile();

  Color:=gci_context.ColorLight;
  {$IFDEF VIRTUALTREES}
  tvSaVie .NodeDataSize := Sizeof(TVieTree)+1;
  {$ENDIF}
  fCanDeletionTV:=false;
  bBloque:=true;
  fFormIndividu:=TFIndividu(Owner);
  lNom.PopupMenu:=FFormIndividu.pmNom;
  lNom.OnMouseEnter:=FFormIndividu.lNomMouseEnter;
  lNom.OnMouseLeave:=FFormIndividu.lNomMouseLeave;
  lNom.OnMouseMove:=FFormIndividu.lNomMouseMove;
  lNom.OnClick:=FFormIndividu.lNomClick;
  inherited;
end;

{$IFDEF VIRTUALTREES}
procedure TFSaVie.tvSaVieGetText(Sender: TBaseVirtualTree; Node: PVirtualNode;
  Column: TColumnIndex; TextType: TVSTTextType; var CellText: String);
begin
  CellText:=PVieTree(sender.GetNodeData(Node))^.Libelle;
end;
{$ENDIF}

function TFSaVie.doOpenQuerys:boolean;
begin
  doInit(FFormIndividu.CleFiche,FFormIndividu.aFIndividuIdentite.lTitre.Caption);
  result:=True;
end;

procedure TFSaVie.mOuvreDocumentClick(Sender:TObject);
begin
  // Matthieu
  p_CreateAndPreviewReport(tvSaVie, fs_RemplaceMsg(rs_Report_EveryThing_About,[lNom.Caption]));
end;

procedure TFSaVie.pmViePopup(Sender:TObject);
var
  cleID:integer;
begin
  with tvSaVie do
    Acte:=PVieTree({$IFDEF VIRTUALTREES}GetNodeData( FocusedNode ){$ELSE}Selected.Data{$ENDIF})^;
  with Acte do
   Begin
    cleID:=cleID;
  //  mOuvreDocument.Visible:=cleID>0;
    mVisualiseMedia.Visible:=ImageIdx in [4,14,17,18];
    mOuvrirFiche.Visible:=ImageIdx in [24,25,29,30,31];
    mOuvrirFiche.Enabled:=cleID>0;
   end;
end;

procedure TFSaVie.mOuvreClick(Sender:TObject);
begin
  OuvreObjetPointe;
end;
{$IFDEF VIRTUALTREES}
procedure TFSaVie.tvSaVieBeforeCellPaint(Sender: {$IFDEF VIRTUALTREES}TBaseVirtualTree{$ELSE}TTreeview{$ENDIF};
  TargetCanvas: TCanvas; Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}; Column: TColumnIndex;
  CellPaintMode: TVTCellPaintMode; CellRect: TRect; var ContentRect: TRect);
begin
  with Sender,PVieTree(GetNodeData(Node))^, Sender.Canvas.Font do
   Begin
    if sosa=1 then
      Color:=clGreen
    else
      case  Sexe of
        1: Color:=gci_context.ColorHomme;
        2: Color:=gci_context.ColorFemme;
        else
          Color:=clWindowText;
      end;
    if Node = RootNode then
      Style:= [fsBold];
   end;
  if Sender.Selected[Node] then
    Sender.Canvas.Brush.Color:=gci_context.ColorMedium;
end;
{$ENDIF}
procedure TFSaVie.tvSaVieDblClick(Sender:TObject);
begin
  with tvSaVie do
  if assigned ( {$IFDEF VIRTUALTREES}FocusedNode{$ELSE}Selected{$ENDIF} ) Then
   Begin
    Acte:=PVieTree({$IFDEF VIRTUALTREES}GetNodeData(FocusedNode){$ELSE}Selected.Data{$ENDIF})^;
    OuvreObjetPointe;
   end;
end;

procedure TFSaVie.OuvreObjetPointe;
var
  cleID,Onglet
  ,ligne:integer;
  TypeTable:string;
begin
  cleID:=Acte.cleID;
  TypeTable:=Acte.TypeTable;
  Onglet:=Acte.Onglet;
  ligne:=Acte.Ligne;

  FFormIndividu.doActiveOnglet(Onglet);

  if cleID>0 then
  begin
    if (TypeTable='T_UNION')and(ligne>=0) then //sélectionner l'union dans l'onglet Unions
      FFormIndividu.aFIndividuUnion.doGoto(ligne)
    else if TypeTable='EVENEMENTS_IND' then //sélectionner l'événement dans l'onglet Identité
      FFormIndividu.aFIndividuIdentite.doGoto(ligne)
    else if TypeTable='MEDIA_POINTEURS' then //sélectionner le média dans l'onglet Médias
      FFormIndividu.aFIndividuPhotosEtDocs.doGoto(ligne)
    else if TypeTable='ADRESSES_IND' then //sélectionner l'adresse dans l'onglet Domiciles
      FFormIndividu.aFIndividuDomiciles.doGoto(ligne);
  end;
end;

procedure TFSaVie.DoShow;
begin
  inherited;
  with tvSaVie do
  {$IFDEF VIRTUALTREES}
    if ChildCount[RootNode]>0 then
      FocusedNode:=RootNode.FirstChild;
  {$ELSE}
    if Assigned(Items.GetFirstNode) then
      Selected:=Items.GetFirstNode;
  {$ENDIF}
end;

procedure TFSaVie.DoShowFirstTime;
begin
  fCanDeletionTV:=true;
end;

procedure TFSaVie.tvSaVieCustomDrawItem(Sender:TCustomTreeView;
  Node:TTreeNode;State:TCustomDrawState;var DefaultDraw:Boolean);
begin
end;

procedure TFSaVie.mVisualiseMediaClick(Sender:TObject);
begin
  VisualiseMedia(Acte.cleID,dm.ReqSansCheck);
end;

procedure TFSaVie.mOuvrirFicheClick(Sender:TObject);
begin
  dm.individu_clef:=Acte.cleID;
  FMain.Ouvre_Et_Selecte_Individu;
end;

procedure TFSaVie.tvSaVieExpanding(Sender: {$IFDEF VIRTUALTREES}TBaseVirtualTree{$ELSE}TObject{$ENDIF};
  Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}; var Allowed: Boolean);
begin
  with tvSaVie do
   if (Xmouse>(GetNodeLevel(Node)+1)*Indent)and bBloque then
    Allowed:=false;
end;

procedure TFSaVie.tvSaVieMouseMove(Sender:TObject;Shift:TShiftState;X,
  Y:Integer);
var
  Noeud:{$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF};
begin
  Noeud:=tvSaVie.GetNodeAt(X,Y);
  pmVie.AutoPopup:=Noeud<>nil;
  if Noeud<>nil then
    tvSaVie.Cursor:=_CURPOPUP
  else
    tvSaVie.Cursor:=crDefault;
  Xmouse:=X;
end;

procedure TFSaVie.tvSaVieGetImageIndex(Sender: {$IFDEF VIRTUALTREES}TBaseVirtualTree{$ELSE}TObject{$ENDIF};
  Node: {$IFDEF VIRTUALTREES}PVirtualNode; Kind: TVTImageKind; Column: TColumnIndex;
  var Ghosted: Boolean; var ImageIndex: Integer{$ELSE}TTreeNode{$ENDIF});
begin
  {$IFNDEF VIRTUALTREES}
  with Node do
  {$ENDIF}
    ImageIndex:=PVieTree({$IFDEF VIRTUALTREES}Sender.GetNodeData( Node ){$ELSE}Data{$ENDIF})^.ImageIdx;
end;


procedure TFSaVie.tvSaVieMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Noeud:{$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF};
begin
  with tvSaVie do
   Begin
    Noeud:=GetNodeAt(X,Y);
    if Noeud<>nil then
      {$IFDEF VIRTUALTREES}FocusedNode{$ELSE}Selected{$ENDIF}:=Noeud;
   end;
end;

end.

