{-----------------------------------------------------------------------}
{                                                                       }
{           Subprogram Name:                                            }
{           Purpose:          Ancestromania GPL                         }
{           Source Language:  Francais                                  }
{           Auteurs :                                                   }
{    Copyright Matthieu Giroux from 2013 to now (LAZARUS),              }
{    Copyright André Langlet (2003 to 2013),                            }
{    Copyright Philippe Cazaux-Moutou (Old Ancestrologie 2003 GPL)      }
{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Description:                                                }
{           Ancestromania est un Logiciel Libre.                        }
{                                                                       }
{ This program is free software: you can redistribute it and/or modify  }
{ it under the terms of the GNU General Public License as published by  }
{ the Free Software Foundation, either version 3 of the License, or     }
{ (at your option) any later version.                                   }
{                                                                       }
{ This program is distributed in the hope that it will be useful,       }
{ but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{ GNU General Public License for more details.                          }
{                                                                       }
{ You should have received a copy of the GNU General Public License     }
{ along with this program.  If not, see <http://www.gnu.org/licenses/>. }
{-----------------------------------------------------------------------}

{                                                                       }
{-----------------------------------------------------------------------}
{                                                                       }
{           Revision History                                            }
{           v#    ,Date       ,Author Name            ,Description      }
{                                                                       }
{-----------------------------------------------------------------------}

unit u_Form_Maj_Internet;

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}
{$IFDEF WINDOWS}
{$DEFINE HTMLVIEW}
{$ELSE}
{.$DEFINE LAZBRO}
{$ENDIF}

{$DEFINE MultiMediaMissing}

interface

uses
{$IFNDEF FPC}
  ShellApi, OleCtrls, SHDocVw, Windows,
{$ELSE}
  LCLIntf, LCLType,
{$ENDIF}
{$IFDEF HTMLVIEW}
  FramView,HTMLView, Readhtml, HtmlGlobals,
{$ELSE}
{$IFDEF LAZBRO}
  lazbro,
{$ENDIF}
{$ENDIF}
  U_FormAdapt, Controls, Graphics, StdCtrls, ExtCtrls, ComCtrls, Classes, Forms,
  SysUtils, Dialogs, u_buttons_appli, JvXPButtons, ExtJvXPCheckCtrls,
  U_OnFormInfoIni, u_netupdate, IdHTTP,
  u_ancestropictimages;

type

  { TFMajInternet }

  TFMajInternet=class(TF_FormAdapt)
    btnFermer: TFWClose;
    btnMAJ: TJvXPButton;
    ch_test: TJvXPCheckbox;
    cb_Serveurs: TComboBox;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    IdHTTP: TIdHTTP;
    Image1: TIATitle;
    Label8: TLabel;
    Label9: TLabel;
    lBuild: TLabel;
    lDate: TLabel;
    lDateAncien: TLabel;
    lSite: TLabel;
    lTailleDistant: TLabel;
    NetUpdate: TNetUpdate;
    OnFormInfoIni: TOnFormInfoIni;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    Panel4: TPanel;
    Panel5: TPanel;
    Panel9:TPanel;
    Image2:TImage;
    Label2:TLabel;
    Panel10:TPanel;
    TabInternet:TPageControl;
    TabMaj:TTabSheet;
    TabNews:TTabSheet;
    lRun:TLabel;
    lAlerte:TLabel;
    Gauge:TProgressBar;
    pConnect:TGroupBox;
    Label1:TLabel;
    lTaille:TLabel;
    Label3:TLabel;
    lReste:TLabel;
    {$IFDEF HTMLVIEW}
    Browser : TFrameViewer;
    {$ELSE}
    {$IFDEF LAZBRO}
    Browser : TLazbro;
    {$ELSE}
    Browser : TMemo;
    {$ENDIF}
    {$ENDIF}
    procedure cb_ServeursChange(Sender: TObject);
    procedure cb_ServeursCloseUp(Sender: TObject);
    procedure ch_testClick(Sender: TObject);
    procedure NetUpdateDownloaded(const Sender: TObject; const TheFile: string;
      const TheStep: TUpdateStep);
    procedure NetUpdateDownloading(const Sender: TObject;
      const Step: TUpdateStep);
    procedure NetUpdateErrorMessage(const Sender: TObject;
      const ErrorCode: integer; const ErrorMessage: string);
    procedure NetUpdateFileDownloaded(const Sender: TObject;
      const MD5OK: Boolean);
    procedure NetUpdateIniRead(const Sender: TObject; const Total: integer);
    procedure NetUpdatePageDownloaded(const Sender: TObject;
      const MD5OK: Boolean);
    procedure btnMAJClick(Sender:TObject);
    procedure btnFermerClick(Sender:TObject);
    procedure TabInternetDrawTabEx(AControl:TCustomTabControl;
      ATab:TTabSheet;Font:TFont);
    {$IFDEF HTMLVIEW}
    procedure WebBrowserHotSpotClick(Sender: TObject; const Target, URL: ThtString;
      var Handled: boolean);
    procedure WebBrowserHotSpotCovered(Sender: TObject; const Target, URL: ThtString);
    procedure WebBrowserLink(Sender: TObject; const Rel, Rev, Href: THtString);
    {$ELSE}
    {$IFDEF LAZBRO}
    procedure WebBrowserLink(const Sender: TLazbro);
    {$ENDIF}
    {$ENDIF}
    procedure p_AfterDownloadExe;
  private
    bOK,bExeEtBaseAjour:Boolean;
    sDate:string;
    iBaseEnCours:integer;
    iExeEnCours:int64;
    procedure ErrorUpdateMessage;
  protected
    procedure DoShow; override;
    procedure DoClose(var Action: TCloseAction); override;
    procedure ControlKeyDown(var Key:Word;
      Shift:TShiftState); override;
  public
    procedure Loaded; override;

  end;

implementation

uses u_dm,
     u_common_const,
     u_common_resources,
     u_genealogy_context,
     u_common_functions,
     u_common_ancestro,
     LazUTF8,
     StrUtils,
     LazFileUtils, //md5Api,
     fonctions_net,
     fonctions_dialogs,
     u_common_ancestro_functions,
     u_form_main,
     unite_messages,
     fonctions_ini,
     fonctions_system,
     fonctions_string;

{$IFDEF FPC}{$R *.lfm}{$ELSE}{$R *.DFM}{$ENDIF}

procedure TFMajInternet.cb_ServeursChange(Sender: TObject);
begin
  NetUpdate.URLBase:=fs_geturlMajAuto(ch_test.checked,cb_Serveurs.ItemIndex);
end;

procedure TFMajInternet.cb_ServeursCloseUp(Sender: TObject);
begin
  btnmaj.caption:=rs_Ini_Download_Again;
  btnmaj.enabled:=True;
end;

procedure TFMajInternet.ch_testClick(Sender: TObject);
begin
  NetUpdate.URLBase:=fs_geturlMajAuto(ch_test.checked,cb_Serveurs.ItemIndex);
  btnmaj.caption:=rs_Ini_Download_Again;
  btnmaj.enabled:=True;
end;


procedure TFMajInternet.NetUpdateDownloaded(const Sender: TObject;
  const TheFile: string; const TheStep: TUpdateStep);
begin
  btnFermer.Enabled:=True;
end;

procedure TFMajInternet.NetUpdateDownloading(const Sender: TObject;
  const Step: TUpdateStep);
Begin
  bOk:=True;
  btnMAJ.Enabled:=False;
  btnFermer.Enabled:=false;
  if (btnMAJ.Caption=gs_Download_update) then
  begin
    pConnect.Visible:=True;
    Gauge.Cursor:=crHourGlass;
    lRun.Visible:=True;
    lRun.Caption:=fs_RemplaceMsg(gs_Downloading_in_progress,[NetUpdate.FileUpdate]);
    Gauge.Visible:=True;
    
  end ;

end;

procedure TFMajInternet.NetUpdateErrorMessage(const Sender: TObject;
  const ErrorCode: integer; const ErrorMessage: string);
begin
  btnFermer.Enabled:=True;
  ErrorUpdateMessage;
end;

procedure TFMajInternet.ErrorUpdateMessage;
begin
  MyMessageDlg(rs_Error_MD5,mmtError, [mmbOk],Self);
  lRun.Visible:=false;
  lReste.Caption:='';
  Gauge.Visible:=false;
  pConnect.Visible:=false;
end;

procedure TFMajInternet.NetUpdateFileDownloaded(const Sender: TObject;
  const MD5OK: Boolean);
begin
  lRun.Caption:=gs_Download_finished;
  // matthieu : MD5Hash problem
  with NetUpdate do
  if MD5OK then
    begin
      btnMAJ.Caption:=rs_Update;
      bok:=True;
      if bExeEtBaseAjour then
        MyMessageDlg(rs_File_Downloaded,mmtInformation, [mmbOk],Self);
    end
  else
    begin
      DeleteFileUTF8(UpdateDir+FileUpdate);
      ErrorUpdateMessage;
      bok:=false;
    end;
  if bOk and(not bExeEtBaseAjour) then
    begin
      pConnect.Visible:=False;
      if MyMessageDlg(rs_Ancestromania_will_be_updated_and_saved
        ,mmtConfirmation, [mmbCancel,mmbOK],Self)=mrCancel then
      begin
        btnMAJ.Enabled:=true;
        exit;
      end;
      with NetUpdate do
        p_OpenFileOrDirectory(UpdateDir+FileUpdate);
      FMain.Close;
     end;
  if not(bExeEtBaseAjour and bOK) then
    btnMAJ.Enabled:=true;
end;

procedure TFMajInternet.NetUpdateIniRead(const Sender: TObject;
  const Total: integer);
begin
  with NetUpdate do
   Begin
    if fi64_VersionExeInt64(VersionExeUpdate)>iExeEnCours then
      begin
        MyMessageDlg(fs_RemplaceMsg(rs_Ancestromania_is_not_fully_updated,[VersionExeUpdate])+
         _CRLF+_CRLF+rs_Ancestromania_should_be_updated,mmtWarning, [mmbOk],Self);
        bOK:=True;
      end
      else if fi_BaseVersionToInt(VersionBaseUpdate)>iBaseEnCours then
      begin
        MyMessageDlg(fs_RemplaceMsg(rs_Ancestromania_is_not_fully_updated,[VersionBaseUpdate])+
         _CRLF+_CRLF+rs_Ancestromania_should_be_updated,mmtWarning, [mmbOk],Self);
        bOK:=True;
      end
      else
      begin
        MyMessageDlg(rs_Ancestromania_is_fully_updated,mmtInformation, [mmbOk],Self);
        bExeEtBaseAjour:=true;
      end;

      TabNews.TabVisible:=True;
      if bOK then
      begin
        with NetUpdate do
        if MD5File<>MD5Ini then
          btnMAJ.Caption:=gs_Download_update
        else
          btnMAJ.Caption:=rs_Update;
        btnMAJ.Enabled:=True;
        btnMAJ.SetFocus;
        lDate.Caption:=sDate;
      end
      else
      begin
        btnMAJ.Enabled:=False;
      end;
      TabInternet.ActivePage:=TabNews;
      
   end;

end;

procedure TFMajInternet.NetUpdatePageDownloaded(const Sender: TObject;
  const MD5OK: Boolean);
begin
  with NetUpdate do
  try
    {$IFDEF HTMLVIEW}
//    Browser.Base := UpdateDir + FilePage;
    {$ELSE}
//      Browser.BaseUrl := UpdateDir + FilePage;
    {$ENDIF}
    Browser.{$IFNDEF WINDOWS}Lines.{$ENDIF}LoadFromFile ( UpdateDir + FilePage );
    TabNews.TabVisible:=True;
    if bOK then
    begin
      if md5ok then
       btnMAJ.Caption:=rs_Update
      else
       btnMAJ.Caption:=gs_Download_update;
      btnMAJ.Enabled:=True;
      btnMAJ.SetFocus;
      lDate.Caption:=sDate;
    end
    else
    begin
      btnMAJ.Enabled:=False;
    end;
    TabInternet.ActivePage:=TabNews;
    

  Except
      MyMessageDlg(rs_Error_Cannot_load_corrupted_file,mmtError, [mmbOk],Self);

  end;
end;


procedure TFMajInternet.ControlKeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  case Key of
    VK_ESCAPE:ModalResult:=mrOk;
  end;
end;

procedure TFMajInternet.DoShow;
Begin
  inherited;
  NetUpdate.URLBase:=fs_geturlMajAuto(ch_test.checked);
end;

procedure TFMajInternet.Loaded;
var
  SR:TSearchRec;
  VersionCur,VersionMax:string;
  lat_ArchitectureType : TArchitectureType;
  LCLType : TLCLType;
begin
  f_GetMemIniFile();
  Browser := {$IFDEF HTMLVIEW}TFrameViewer{$ELSE}{$IFDEF LAZBRO}TLazbro{$ELSE}TMemo{$ENDIF}{$ENDIF}.Create(Self);
  Browser.Parent:=TabNews;
  Browser.Align:=alClient;
  Browser.Visible:=True;
  {$IFDEF HTMLVIEW}
  Browser.OnLink:=WebBrowserLink;
  Browser.OnHotSpotTargetClick  :=WebBrowserHotSpotClick;
  Browser.OnHotSpotTargetCovered:=WebBrowserHotSpotCovered;
  {$ELSE}
  {$IFDEF LAZBRO}
  Browser.OnUrlChange:=WebBrowserLink;
  {$ENDIF}
  {$ENDIF}
  Color:=gci_context.ColorLight;
  LCLType := ftl_GetLCLCType;
  with NetUpdate do
   Begin
    UpdateDir:=sUpdateDir+'Updates' +DirectorySeparator;
    lat_ArchitectureType := fat_GetArchitectureType;
    FileIni:=fs_GetIniFileNameUpdate(lat_ArchitectureType,fpt_GetPackagesType,CST_PROCESSOR_TYPE,LCLType,IniFilename);
    FilePage:= WebPage;
    FileUpdate:=fs_GetFileNameUpdate(lat_ArchitectureType,fpt_GetPackagesType(FileExistsUTF8(GetAppDir+CST_PARANCES)),CST_PROCESSOR_TYPE,LCLType,'Update_Ancestro');
   end;

  Panel10.Color:=gci_context.ColorDark;
  TabNews.TabVisible:=False;
  lBuild.Caption:=fs_RemplaceMsg(rs_Caption_Exe_database, [NumVersionCourt(0,gr_ExeVersion),sVersionBase]);
  lDateAncien.Caption:=rs_from+' : '+fs_DateTime2Str(FileDateToDateTime(FileAgeUTF8(Application.ExeName) ));
  bExeEtBaseAjour:=False;
  VersionMax:='';
  try
    if DirectoryExistsUTF8(gci_context.Path_Applis_Ancestro+ScriptsDir) then
    begin
      if FindFirstUTF8(gci_context.Path_Applis_Ancestro+ScriptsDir+DirectorySeparator+'*.sql', faAnyFile, SR) { *Converted from FindFirstUTF8*  }=0 then
      begin
        VersionCur:=RightStr(extractFileNameWithoutExt(gci_context.Path_Applis_Ancestro+ScriptsDir+DirectorySeparator+SR.Name),4);
        if StrToIntDef(VersionCur,-1)<>-1 then
          VersionMax:=VersionCur;
        while FindNextUTF8(SR) { *Converted from FindNextUTF8*  }=0 do
        begin
          VersionCur:=RightStr(extractFileNameWithoutExt(gci_context.Path_Applis_Ancestro+ScriptsDir+DirectorySeparator+SR.Name),4);
          if (VersionCur>VersionMax)and(StrToIntDef(VersionCur,-1)<>-1) then
            VersionMax:=VersionCur;
        end;
      end;
      FindCloseUTF8(SR); { *Converted from FindCloseUTF8*  }
    end;
    try
      if VersionMax>'' then
      begin
        VersionMax:=LeftStr(VersionMax,Length(VersionMax)-3)+'.'+RightStr(VersionMax,3);
        iBaseEnCours:=fi_BaseVersionToInt (VersionMax);
      end
      else
        iBaseEnCours:=0;
    except
      iBaseEnCours:=0;
    end;

    iExeEnCours:=fi64_VersionExeInt64;
  except
    on E:Exception do
    begin
      MyShowMessage(rs_Error_Initializing_Internet_Update+_CRLF+_CRLF+E.Message);
      Close;
    end;
  end;
  inherited;
end;

procedure TFMajInternet.btnMAJClick(Sender:TObject);
begin
  btnMAJ.Enabled:=false;
  

  bOK:=False;

  lAlerte.Visible:=true;

  if (btnMAJ.Caption=gs_Download_update)or(btnMAJ.Caption=rs_Update) then
  begin
    TabInternet.ActivePage:=TabMaj;
    NetUpdate.Update;
  end
  else
  begin
    screen.Cursor:=crHourGlass;
    lRun.Visible:=True;

    

    lRun.Caption:=rs_Caption_Verify_updates;

    NetUpdate.UpdateIniPage;

  end;
end;

procedure TFMajInternet.DoClose(
  var Action:TCloseAction);
begin
  
//  Action:=caFree; //libérée par la fonction qui l'appelle
end;

procedure TFMajInternet.btnFermerClick(Sender:TObject);
begin
  //WebBrowser.Clear;
  Close;
end;

procedure TFMajInternet.TabInternetDrawTabEx(AControl:TCustomTabControl;
  ATab:TTabSheet;Font:TFont);
begin
  Font.Color:=gci_context.ColorTexteOnglets;
  if ATab.PageIndex = AControl.PageIndex then
    ATab.Color:=gci_context.ColorLight
  else
    ATab.Color:=gci_context.ColorDark;
end;

procedure TFMajInternet.p_AfterDownloadExe;
begin

end;

{$IFDEF HTMLVIEW}

procedure TFMajInternet.WebBrowserHotSpotClick(Sender: TObject;
  const Target, URL: ThtString; var Handled: boolean);
{This routine handles what happens when a hot spot is clicked.  The assumption
 is made that DOS filenames are being used. .EXE, .WAV, .MID, and .AVI files are
 handled here, but other file types could be easily added.

 If the URL is handled here, set Handled to True.  If not handled here, set it
 to False and ThtmlViewer will handle it.}
{$ifndef MultiMediaMissing}
const
  snd_Async = $0001;  { play asynchronously }
{$endif}
var
  {$ifndef MultiMediaMissing}
  PC: array[0..255] of {$ifdef UNICODE} WideChar {$else} AnsiChar {$endif};
  {$endif}
  S, Params: ThtString;
  Ext: string;
  I, J, K: integer;
begin
  Handled := False;

  {check for various file types}
  I := Pos(':', URL);
  J := Pos('FILE:', UpperCase(URL));
  if (I <= 2) or (J > 0) then
  begin                      {apparently the URL is a filename}
    S := URL;
    K := Pos(' ', S);     {look for parameters}
    if K = 0 then K := Pos('?', S);  {could be '?x,y' , etc}
    if K > 0 then
    begin
      Params := Copy(S, K+1, 255); {save any parameters}
      setLength(S, K-1);            {truncate S}
    end
    else
      Params := '';
    S := (Sender as TFrameViewer).HTMLExpandFileName(S);
    Ext := Uppercase(ExtractFileExt(S));
    if Ext = '.WAV' then
    begin
      Handled := True;
  {$ifndef MultiMediaMissing}
     sndPlaySound(StrPCopy(PC, S), snd_ASync);
  {$endif}
    end
    else if Ext = '.EXE' then
    begin
      Handled := True;
      fs_executeprocess(S, ' ' + Params, False,[]);
    end
    else if (Ext = '.MID') or (Ext = '.AVI')  then
    begin
      Handled := True;
      fs_executeprocess('MPlayer.exe',' /play /close ' + S, False,[]);
    end;
    {else ignore other extensions}
//    Edit2.Text := URL;
    Exit;
  end;

  I := Pos('MAILTO:', UpperCase(URL));
  J := Pos('HTTP://', UpperCase(URL));
  if (I > 0) or (J > 0) then
  begin
    {Note: ShellExecute causes problems when run from Delphi 4 IDE}
    p_OpenFileOrDirectory( URL);
    Handled := True;
    Exit;
  end;

//  Edit2.Text := URL;   {other protocall}
end;

procedure TFMajInternet.WebBrowserHotSpotCovered(Sender: TObject;
  const Target, URL: ThtString);
{mouse moved over or away from a hot spot.  Change the status line}
var
  Text: ThtString;
begin
  if URL = '' then
    Text := ''
  else if Target <> '' then
    Text := 'Target: '+Target+'  URL: '+URL
  else
    Text := 'URL: '+URL;

{$ifdef UseTNT}
  TntLabel.Caption := Text;
{$else}
//  InfoPanel.Caption := Text;
{$endif}
end;

procedure TFMajInternet.WebBrowserLink(Sender: TObject; const Rel, Rev,
  Href: THtString);
begin
  p_OpenFileOrDirectory(Href);
end;

{$ELSE}
{$IFDEF LAZBRO}
procedure TFMajInternet.WebBrowserLink(const Sender: TLazbro);
begin
  p_OpenFileOrDirectory(Sender.BaseUrl);
end;

{$ELSE}
{$ENDIF}
{$ENDIF}


end.

